/*******************************************************************

**This SAS file contains the program used to create two additional datasets that serve as input to construc the final datasets:
1. a dataset used as an input for the construction of the variable Optimism;
2. a dataset used as an input for the construction of the variable Delta.

*******************************************************************/

*PATH;
LIBNAME RAW 'D:\IBES\RAW';
LIBNAME RESULT "D:\IBES\RESULT";

/*********************************************************************************/
*
*1. a dataset used as an input for the construction of the variable Optimism                            ;
*
************************************************************************************;

*GET ALL QUARTERLY EARNINGS (EPS) ESTIMATES;
DATA DET_1 
(KEEP = TICKER ANALYS FPEDATS EST ANNDATS REVDATS FPI ESTIMATOR ANNTIMS ACTDATS);
SET RAW.IBES_DETAIL_QTR;
EST = VALUE;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
IF TICKER NE "";
IF ANALYS NE .;
IF EST NE .;
RUN;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS DESCENDING ANNTIMS; QUIT;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS; QUIT;

*GET ALL ACTUAL EARNINGS (EPS) ANNOUNCEMENTS;
DATA ACT_0;
SET RAW.ACTUAL;
FPEDATS = PENDS;
IF PDICITY = 'QTR';
IF MEASURE = 'EPS';
IF VALUE NE .;
IF ANNDATS NE .;
FORMAT FPEDATS YYMMDDN8.;
RUN;
PROC SORT DATA = ACT_0 NODUPKEY; BY TICKER FPEDATS; QUIT;

/*MERGE IN PERMNO*/
DATA RET; SET RAW.DAILY_RET;
KEEP NCUSIP PERMNO PRC DATE;
RUN;
PROC SORT DATA = RET NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE ACT_PERMNO AS
SELECT A.TICKER, A.ANNDATS, B.PERMNO, B.DATE AS CRSP_DATE, B.PRC
FROM ACT_0 A LEFT JOIN RET B
ON A.CUSIP = B.NCUSIP AND 0 <= A.ANNDATS - B.DATE <= 4;
QUIT;
DATA ACT_PERMNO_0; SET ACT_PERMNO; IF NOT MISSING(PERMNO); RUN;
PROC SORT DATA = ACT_PERMNO_0; BY TICKER ANNDATS CRSP_DATE; QUIT;
PROC SORT DATA = ACT_PERMNO_0 OUT = ACT_PERMNO_1 NODUPKEY; BY TICKER ANNDATS; QUIT;

PROC SQL;
CREATE TABLE ACT_1 AS
SELECT A.*, B.PERMNO, B.PRC AS EA_PRC
FROM ACT_0 A LEFT JOIN ACT_PERMNO_1 B
ON A.TICKER = B.TICKER AND A.ANNDATS = B.ANNDATS;
QUIT;

*MERGE IN LAST QTR'S EARNINGS (EPS) ANNOUNCEMENT;
PROC SQL;
CREATE TABLE ACT_2 AS
SELECT A.TICKER, A.CUSIP, A.PERMNO, A.USFIRM, 
A.VALUE AS CURR_ACT, A.FPEDATS AS CURR_FPE, A.ANNDATS AS CURR_ANN, 
B.VALUE AS PREV_ACT, B.FPEDATS AS PREV_FPE, B.EA_PRC AS PRC, B.ANNDATS AS PREV_ANN
FROM ACT_1 AS A, ACT_1 AS B
WHERE A.TICKER = B.TICKER 
AND  0 < A.FPEDATS - B.FPEDATS <= 110
AND  A.ANNDATS > B.ANNDATS;
QUIT; 
PROC SORT DATA = ACT_2; BY TICKER CURR_FPE CURR_ANN DESCENDING PREV_FPE;QUIT;
PROC SORT DATA = ACT_2 OUT = ACT_21 NODUPKEY; BY TICKER CURR_FPE CURR_ANN;QUIT;

*MERGE ANALYST FORECASTS WITH ACTUAL EPS;
PROC SQL;
CREATE TABLE DET_2 AS
SELECT A.*, 
B.ESTIMATOR, B.ANALYS, B.EST AS CURR_EST_FIN, 
B.ANNDATS AS CURR_EST_FIN_ANN, B.REVDATS, B.ACTDATS
FROM ACT_21 AS A, DET_1 AS B
WHERE A.TICKER = B.TICKER 
AND A.CURR_FPE = B.FPEDATS
AND A.CURR_ANN > B.ANNDATS;
QUIT;

*IDENTIFY EXCLUDED ESTIMATES;
DATA EXCLUDED; SET RAW.EXCLUDED;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
RUN;

PROC SQL;
CREATE TABLE DET_3 AS
SELECT A.*, B.EXCFLA = 'X' AS FL_EXCL, B.EXCDATS, B.ACTDATS AS ACTDATS_EXCL, B.VALUE AS VALUE_EXCL, B.EXCENDS
FROM DET_2  A LEFT JOIN EXCLUDED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS 
AND A.CURR_FPE = B.FPEDATS AND B.EXCENDS GT A.CURR_ANN AND A.ACTDATS = B.ACTDATS AND A.CURR_EST_FIN = B.VALUE;
QUIT;

PROC SQL;
CREATE TABLE DET_4 AS
SELECT A.*, B.ASTPDATS NE . AS FL_STOP, B.ASTPDATS
FROM DET_3 A LEFT JOIN RAW.STOPPED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.CURR_FPE = B.FPEDATS AND B.ASTPDATS LE A.CURR_ANN
AND A.CURR_EST_FIN_ANN LE B.ASTPDATS AND B.PDICITY='Q' AND B.MEASURE = "EPS";
QUIT;

PROC SORT DATA = DET_4; BY TICKER CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN DESCENDING FL_EXCL DESCENDING FL_STOP DESCENDING ASTPDATS; QUIT;
PROC SORT DATA = DET_4 NODUPKEY; BY TICKER CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN; QUIT; 

DATA DET_5; SET DET_4;
IF EXCDATS = . THEN DO; IF ASTPDATS NE . THEN DELETE;END;
IF EXCDATS NE . THEN DO;   IF ASTPDATS NE . THEN DO;
   IF ASTPDATS LE EXCDATS THEN DELETE;END;END;
IF EXCDATS GE CURR_ANN THEN FL_EXCL = 0;
IF CURR_EST_FIN_ANN GE CURR_ANN THEN DELETE;
RUN;

/*adjust stock splits*/
DATA RET; SET RAW.DAILY_RET;
KEEP PERMNO CFACSHR DATE;
RUN;
PROC SORT DATA = RET NODUPREC; BY _ALL_; QUIT;

PROC SORT DATA = DET_5 (KEEP = TICKER PERMNO CURR_FPE CURR_ANN) OUT = ACTUAL_ADJ0 NODUPKEY;
BY  TICKER PERMNO CURR_FPE CURR_ANN;
QUIT;

PROC SQL;
CREATE TABLE ACTUAL_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_FPE, A.CURR_ANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM ACTUAL_ADJ0 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_ANN -  B.DATE >= 0;
QUIT;
PROC SORT DATA = ACTUAL_ADJ; BY TICKER PERMNO CURR_FPE CURR_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = ACTUAL_ADJ OUT = ACTUAL_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_FPE CURR_ANN; QUIT;

PROC SQL;
CREATE TABLE DET_51 AS
SELECT A.*, B.CFACSHR AS ACT_ADJ
FROM DET_5 A LEFT JOIN ACTUAL_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.CURR_ANN = B.CURR_ANN;
QUIT;

PROC SQL;
CREATE TABLE DET_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM DET_51 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_EST_FIN_ANN -  B.DATE >= 0;
QUIT;

PROC SORT DATA = DET_ADJ; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = DET_ADJ OUT = DET_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN;QUIT;

PROC SQL;
CREATE TABLE DET_52 AS
SELECT A.*, B.CFACSHR AS DET_ADJ
FROM DET_51 A LEFT JOIN DET_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND A.CURR_EST_FIN_ANN = B.CURR_EST_FIN_ANN;
QUIT;

DATA RESULT.SE_190516; SET DET_52;
IF MISSING(ACT_ADJ) = 0 AND MISSING(DET_ADJ) = 0 AND (ACT_ADJ NE DET_ADJ) THEN DO;
	CURR_EST_FIN = CURR_EST_FIN*(ACT_ADJ/DET_ADJ);
END;
RUN;


/*********************************************************************************/
*
*2. a dataset used as an input for the construction of the variable Delta                           ;
*
************************************************************************************;

/** program reads in data from execcomp and calculates         **/ 
/** delta**/

%let wrds=wrds-cloud.wharton.upenn.edu 4016;
options comamid=TCP remote=WRDS;
signon username=_prompt_;

libname result 'D:\IBES\RESULT_202008';
options nolabel nocenter;

rsubmit;

%let start_year=1992;
%let end_year=2015;

* Extract Compustat data;
data comp1;
set comp.funda;
where &start_year-6 <= fyear <= &end_year
and indfmt='INDL' and datafmt='STD' and popsrc='D' and consol='C';
fyenddt=datadate;
fybegdt=intnx('month', fyenddt, -11, 'beg');
format fyenddt fybegdt date9.;
keep gvkey datadate fyear fybegdt fyenddt;
rename fyear=year;
run;

proc sort data=comp1; by gvkey datadate; run;

* Link GVKEYS to CRSP identifiers;
proc sql;
create table combined_for_wrds
as select a.*, b.lpermno as permno
from comp1 as a, crsp.ccmxpf_lnkhist as b
where a.gvkey=b.gvkey and b.linkprim in ('P', 'C') and
b.LINKTYPE in ('LU', 'LC') and
(b.LINKDT <= a.fyenddt or missing(b.LINKDT)) and (a.fyenddt <= b.LINKENDDT or missing(b.LINKENDDT))
order by gvkey, datadate;
quit;

* Add CRSP monthly return data;
proc sql;
create table crsp1 as select a.*, b.date, b.ret label=''
from combined_for_wrds as a, crsp.msf as b
where a.permno = b.permno and
intck('month', b.date, a.fybegdt) between 1 and 60
order by gvkey, datadate, date;
quit;

* Calculate monthly stock return volatility;
proc means data=crsp1 noprint;
var ret;
by gvkey datadate year;
output out=exec_roll_vol_fyear n=nrollingstd std=rollingstd;
run;

* Recreate riskfree_rate table;
data rates;
set frb.rates_daily;
year=year(date);
if &start_year <= year <= &end_year;
keep date year TCMNOM_Y1 TCMNOM_Y2 TCMNOM_Y3 TCMNOM_Y5 TCMNOM_Y7 TCMNOM_Y10;
run;

proc means data=rates noprint;
var TCMNOM_Y1 TCMNOM_Y2 TCMNOM_Y3 TCMNOM_Y5 TCMNOM_Y7 TCMNOM_Y10;
by year;
output out=rates_annual mean=oneyr twoyr threeyr fiveyr sevenyr tenyr;
run;

data riskfree_rate (drop=_TYPE_ _FREQ_);
set rates_annual;
oneyr=round(oneyr, 0.01);
twoyr=round(twoyr, 0.01);
threeyr=round(threeyr, 0.01);
fiveyr=round(fiveyr, 0.01);
sevenyr=round(sevenyr, 0.01);
tenyr=round(tenyr, 0.01);
fouryr=round(threeyr+(fiveyr-threeyr)/2, 0.01);
sixyr=round(fiveyr+(sevenyr-fiveyr)/2, 0.01);
eightyr=round(sevenyr+(tenyr-sevenyr)/3, 0.01);
nineyr=round(sevenyr+(tenyr-sevenyr)/3*2, 0.01);
run;

data anncomp;
set execcomp.anncomp;
where &start_year <= year <= &end_year;
coperol=co_per_rol;
shrown=SHROWN_EXCL_OPTS;
shrowntotal=SHROWN_TOT;
opts_unvested_num=OPT_UNEX_UNEXER_NUM;
opts_unvested_val=OPT_UNEX_UNEXER_EST_VAL;
opts_vested_num=OPT_UNEX_EXER_NUM;
opts_vested_val=OPT_UNEX_EXER_EST_VAL;
opts_exercised_num= OPT_EXER_NUM;
keep gvkey coperol year execid allothpd allothtot bonus 
eip_unearn_num eip_unearn_val noneq_incent old_datafmt_flag
OPTION_AWARDS_BLK_VALUE 
OPTION_AWARDS_FV OPTION_AWARDS_NUM OPTION_AWARDS_RPT_VALUE 
OPTION_AWARDS  opts_exercised_num
opts_vested_val 
opts_vested_num opts_unvested_val
opts_unvested_num  OTHANN OTHCOMP  
RSTKGRNT   SALARY   
SHROWN  shrowntotal  SHRS_VEST_NUM 
SHRS_VEST_VAL  STOCK_AWARDS_FV  STOCK_AWARDS 
STOCK_UNVEST_NUM  STOCK_UNVEST_VAL 
TDC1  TITLEANN ltip ceoann defer_rpt_as_comp_tot;
proc sort nodupkey;
by coperol year;
run;

data person;
set execcomp.person;
keep execid exec_fullname ;
proc sort nodupkey;
by execid;
run;

proc sql;
create table anncomp1
as select *
from anncomp x left join person y 
on x.execid=y.execid;

proc sort nodupkey data=anncomp1;
by coperol year;
run;

data codirfin;
set execcomp.codirfin (keep = gvkey year bs_volatility bs_yield divyield shrsout prccf fyr srcdate);
execcomp_volatility = bs_volatility;
execcomp_yield=bs_yield/100;
if gvkey="001239" and year=2007 then divyield=0.66;
divyield=divyield/100;
keep gvkey year divyield execcomp_volatility execcomp_yield prccf shrsout srcdate fyr;
proc sort nodupkey;
by gvkey year;
run;

proc sort data=codirfin;
by gvkey year;
data codirfin;
set codirfin;
lgvkey=lag(gvkey);
lyear=lag(year)+1;
l2gvkey=lag2(gvkey);
l2year=lag2(year)+2;
ldivyield=lag(divyield);
l2divyield=lag2(divyield);
if gvkey^=lgvkey or year^=lyear then ldivyield=.;
if gvkey^=l2gvkey or year^=l2year then l2divyield=.;
if divyield^=. and ldivyield^=. and l2divyield^=. 
then estimated_yield= (divyield+ldivyield+l2divyield)/3;
if divyield^=. and ldivyield^=. and l2divyield=. 
then estimated_yield= (divyield+ldivyield)/2;
if divyield^=. and ldivyield=. and l2divyield=. 
then estimated_yield= divyield;
if divyield=. and ldivyield=. and l2divyield=. 
then estimated_yield=.;
drop ldivyield l2divyield lgvkey l2gvkey lyear l2year;
run;

data combined;
set combined_for_wrds(keep=gvkey permno year fybegdt fyenddt);
where &start_year <= year <=&end_year;
proc sort nodupkey;
by gvkey year;
run;

proc sql;
create table codirfin1
as select *
from codirfin x left join combined y 
on  x.gvkey=y.gvkey and x.year=y.year;

proc sort nodupkey data=codirfin1;
by gvkey year;
run;

data anncomp2;
set anncomp1;
keep gvkey coperol year exec_fullname
old_datafmt_flag option_awards_blk_value
option_awards_num opts_exercised_num
opts_vested_num opts_vested_val 
opts_unvested_num opts_unvested_val shrown;  
run;

proc sql;
create table excomp1
as select *
from anncomp2 x left join codirfin1 y 
on x.gvkey=y.gvkey and x.year=y.year;

data excomp2;
set excomp1;
if fybegdt=. or fyenddt=. then delete;
run;

proc sort nodupkey data=excomp2;
by coperol year;
run;

data stgrttab;
set execcomp.stgrttab;
coperol=co_per_rol;
keep coperol year blkshval exdate expric grntnum mktpric numsecur;
proc sort nodupkey;
by coperol year grntnum;
run;

proc sql;
create table excomp3
as select *
from excomp2 x left join stgrttab y 
on x.coperol=y.coperol and x.year=y.year;

proc sort nodupkey data=excomp3;
by coperol year grntnum;
run;

data ex_black;
set execcomp.ex_black;
execcomp_rf=risk_free_rate/100;
keep execcomp_rf year;
run;

proc sql;
create table excomp4
as select *
from excomp3 x left join ex_black y 
on  x.year=y.year;

data volatility;
set exec_roll_vol_fyear;
where &start_year <= year <= &end_year;
estimated_volatility=rollingstd*(12**0.5);
proc sort nodupkey;
by gvkey year;
run;

proc sql;
create table BS_estimates as
select * 
from volatility x left join riskfree_rate y
on x.year=y.year;

data BS_estimates1;
set BS_estimates;
array X 
oneyr twoyr threeyr fouryr fiveyr sixyr sevenyr eightyr nineyr tenyr;
do over X;
X=X/100;
end;
keep estimated_volatility   oneyr twoyr threeyr fouryr fiveyr sixyr 
sevenyr eightyr nineyr tenyr gvkey year nrollingstd;
run;

proc sort nodupkey data=BS_estimates1;
by gvkey year;
run;

proc sql;
create table excomp5 as
select *
from excomp4 x left join BS_estimates1 y
on x.gvkey=y.gvkey and x.year=y.year;

proc sort data=excomp5;
by year;
proc univariate data=excomp5 noprint;
by year;
var estimated_yield estimated_volatility;
output out=out1 pctlpts=5 95 pctlpre=estimated_yield estimated_volatility;

data out1;
set out1;
keep year estimated_yield5 estimated_yield95 estimated_volatility5 estimated_volatility95;
run;

proc sort data=excomp5;
by year;
proc sort data=out1;
by year;
data excomp6a;
merge excomp5 (in=A) out1 (in=B);
by year;
if A;
run;

data excomp6b;
set excomp6a;
if estimated_yield>estimated_yield95 and estimated_yield^=. then estimated_yield=estimated_yield95;
if estimated_volatility>estimated_volatility95 and estimated_volatility^=. then estimated_volatility=estimated_volatility95;
if estimated_yield<estimated_yield5 and estimated_yield^=. then estimated_yield=estimated_yield5;
if estimated_volatility<estimated_volatility5 and estimated_volatility^=. then estimated_volatility=estimated_volatility5;
drop estimated_volatility5 estimated_volatility95 estimated_yield5 estimated_yield95;
run;

proc sql;
create table excomp6c as
select *, mean(estimated_volatility) as mean_vol, mean(estimated_yield) as mean_yield
from excomp6b
group by year;

data excomp7;
set excomp6c;
format exdate mmddyy10.;
if nrollingstd<12 and nrollingstd^=. then estimated_volatility=mean_vol; 
if year<=2006 and estimated_volatility=. then estimated_volatility=execcomp_volatility;
if year<=2006 and estimated_yield=. then estimated_yield=execcomp_yield;
if fyr=6 then assumed_grantyear=year-1;
else assumed_grantyear=year;
assumed_grantdate=mdy(7,1,assumed_grantyear);
maturity_grantdate=(exdate-assumed_grantdate)/365;
maturity_yearend=(exdate-fyenddt)/365;
if coperol=. then delete;
if numsecur^=. and numsecur>0 and 
maturity_yearend<0 and maturity_yearend^=.
then maturity_yearend=0.001; 
drop assumed_grantyear assumed_grantdate;
run;

proc sql;
create table excomp8 as
select *, mean(maturity_grantdate) as avg_mat_grantdate,  mean(maturity_yearend) as avg_mat_yearend
from excomp7
group by coperol, year;

data excomp9;
set excomp8;
if maturity_grantdate=. and avg_mat_grantdate^=. then maturity_grantdate=avg_mat_grantdate;
if maturity_yearend=. and avg_mat_yearend^=. then maturity_yearend=avg_mat_yearend;
run;

data excomp10;
set excomp9;
if round(maturity_yearend,1) =1  then rfc=oneyr;
if round(maturity_yearend,1) =2  then rfc=twoyr;
if round(maturity_yearend,1) =3  then rfc=threeyr;
if round(maturity_yearend,1) =4  then rfc=fouryr;
if round(maturity_yearend,1) =5  then rfc=fiveyr;
if round(maturity_yearend,1) =6  then rfc=sixyr;
if round(maturity_yearend,1) =7  then rfc=sevenyr;
if round(maturity_yearend,1) =8  then rfc=eightyr;
if round(maturity_yearend,1) =9  then rfc=nineyr;
if round(maturity_yearend,1)=10 then rfc=tenyr;
if (maturity_yearend^=. and round(maturity_yearend,1)>10) then rfc=tenyr;
bs_yield=estimated_yield;
sigma=estimated_volatility;
sigmasq=sigma*sigma;
drop execcomp_rf execcomp_yield execcomp_volatility
estimated_yield estimated_volatility;
run;
data excomp11;
set excomp10;
if numsecur=. and (exdate=. and expric=. and mktpric=.
and option_awards_num=0) then numsecur=0;
Xc = expric;
realizable_value = (prccf-Xc)*numsecur;
if realizable_value<0 then realizable_value=0;
if mktpric=. or Xc=. or numsecur=. then realizable_value=.;
Zc_yearend = (log(prccf/Xc)+maturity_yearend*(rfc-bs_yield+sigmasq/2))/(sigma*sqrt(maturity_yearend));
if maturity_yearend=. then Zc_yearend=.;
Zc_grantdate= (log(mktpric/Xc)+maturity_grantdate*(rfc-bs_yield+sigmasq/2))/(sigma*sqrt(maturity_grantdate));
if maturity_grantdate=. then Zc_grantdate=.;
Sc_yearend = exp(-bs_yield*maturity_yearend)*probnorm(Zc_yearend)*numsecur*prccf/100;
if Zc_yearend=. then Sc_yearend=.;
if  numsecur=0 then Sc_yearend=0;
Vc_grantdate = numsecur*(mktpric*exp(-bs_yield* maturity_grantdate)*probnorm(Zc_grantdate)
-Xc*exp(-rfc* maturity_grantdate)*probnorm(Zc_grantdate-sigma*sqrt( maturity_grantdate)));
if Zc_grantdate=. then Vc_grantdate=.;
if numsecur=0 then Vc_grantdate=0;
if Zc_yearend^=. then do;
Vc_yearend = numsecur*(prccf*exp(-bs_yield*maturity_yearend)*probnorm(Zc_yearend)
-Xc*exp(-rfc*maturity_yearend)*probnorm(Zc_yearend-sigma*sqrt(maturity_yearend)));
end;

if numsecur=0 then Vc_yearend=0;

if Zc_yearend^=. then do;
Rc_yearend = exp(-bs_yield* maturity_yearend)*PDF('normal',Zc_yearend,0,1)*prccf*sqrt( maturity_yearend)*0.01*numsecur;
end;
if Zc_yearend=. or maturity_yearend=. then Rc_yearend=.;

if numsecur=0 then Rc_yearend=0;

run;

proc sql;
create table excomp12
as select *, sum(Vc_grantdate) as Vopts_grantdate,
sum(Vc_yearend) as Vopts_yearend,
sum(numsecur) as sumnumsecur,
sum(Sc_yearend) as Sopts_grants_yearend,  
sum(Rc_yearend) as Ropts_grants_yearend, 
sum(realizable_value) as sumrealizable_value
from excomp11
group by coperol,year;
data excomp13;
set excomp12;
if sumnumsecur=0 or sumnumsecur=. then do;
Sopts_grants_yearend=0;
Vopts_grantdate=0;
Ropts_grants_yearend=0;
sumrealizable_value=0;
Vopts_yearend=0;
FVopts_yearend=0;
end;

keep coperol year Vopts_yearend sumnumsecur Sopts_grants_yearend  
Ropts_grants_yearend sumrealizable_value sigma sigmasq bs_yield option_awards_num 
opts_vested_num opts_unvested_num opts_exercised_num
opts_vested_val opts_unvested_val shrown shrsout old_datafmt_flag avg_mat_yearend prccf
fybegdt fyenddt oneyr twoyr threeyr fouryr fiveyr sixyr sevenyr eightyr nineyr tenyr exec_fullname;

proc sort nodupkey;
by coperol year;
run;

proc sort data=excomp13;
by coperol year;
data excomp13a;
set excomp13;
lcoperol=lag(coperol);
lyear=lag(year)+1;
lag_opts_vested_num=lag(opts_vested_num);
lag_opts_unvested_num=lag(opts_unvested_num);
if coperol^=lcoperol or year^=lyear then do;
lag_opts_vested_num=. ; lag_opts_unvested_num=.;
end;
drop lcoperol lyear ;
run;

data excomp13b; 
set excomp13a;

if opts_unvested_num<0 and opts_unvested_num^=. then opts_unvested_num=0;
if opts_unvested_val<0 and opts_unvested_val^=. then opts_unvested_val=0;
if opts_vested_num<0 and opts_vested_num^=. then opts_vested_num=0;
if opts_vested_val<0 and opts_vested_val^=. then opts_vested_val=0;
opts_unvested_num_excl_curryear = opts_unvested_num - option_awards_num;
opts_unvested_val_excl_curryear = opts_unvested_val-sumrealizable_value;
opts_vested_num_excl_curryear =  opts_vested_num;
opts_vested_val_excl_curryear =  opts_vested_val;
if  opts_unvested_num_excl_curryear^=. and opts_unvested_num_excl_curryear<0 
then do;
opts_unvested_num_excl_curryear=0;
opts_unvested_val_excl_curryear =0;
opts_vested_num_excl_curryear= opts_vested_num + (opts_unvested_num - option_awards_num);
opts_vested_val_excl_curryear= opts_vested_val + (opts_unvested_val-sumrealizable_value);
end;
if  opts_unvested_val_excl_curryear^=. and opts_unvested_val_excl_curryear<0 
then  opts_unvested_val_excl_curryear=0;
run;

data excomp13c;
set excomp13b;
if opts_vested_num_excl_curryear<0 and opts_vested_num_excl_curryear^=. then 
if round((opts_unvested_num +  opts_vested_num),0.1)=round((LAG_opts_unvested_num + LAG_opts_vested_num + OPTION_AWARDS_NUM - opts_exercised_num),0.1) 
then do;
opts_vested_num_excl_curryear=0; opts_vested_val_excl_curryear=0;
DUMMY1=1;
end; 

if opts_vested_num_excl_curryear<0 and opts_vested_num_excl_curryear^=. then 
if round((opts_unvested_num +  opts_vested_num),0.1)^=round((LAG_opts_unvested_num + LAG_opts_vested_num + OPTION_AWARDS_NUM - opts_exercised_num),0.1) 
then do;
opts_unvested_num_excl_curryear=.; opts_unvested_val_excl_curryear=.;
opts_vested_num_excl_curryear=.; opts_vested_val_excl_curryear=.;
DUMMY2=1;
end;

if opts_vested_val_excl_curryear<0 and opts_vested_val_excl_curryear^=. 
then opts_vested_val_excl_curryear=0;
run;

data excomp14;
set excomp13c;
if option_awards_num^=. and option_awards_num>0 
then matunvest=avg_mat_yearend-1;
else matunvest=9;
if matunvest<0 and matunvest^=. then matunvest=0.001;
if round(matunvest,1) <=1 then rfunvest=oneyr;
if round(matunvest,1) =2  then rfunvest=twoyr;
if round(matunvest,1) =3  then rfunvest=threeyr;
if round(matunvest,1) =4  then rfunvest=fouryr;
if round(matunvest,1) =5  then rfunvest=fiveyr;
if round(matunvest,1) =6  then rfunvest=sixyr;
if round(matunvest,1) =7  then rfunvest=sevenyr;
if round(matunvest,1) =8  then rfunvest=eightyr;
if round(matunvest,1) =9  then rfunvest=nineyr;
if round(matunvest,1)=10  then rfunvest=tenyr;
if (matunvest^=. and round(matunvest,1)>1) then rfunvest=tenyr;
if opts_unvested_num_excl_curryear^=0 then 
Xunvest=prccf-(opts_unvested_val_excl_curryear/opts_unvested_num_excl_curryear);
if (Xunvest<0 and Xunvest^=.) then Xunvest=0.01;

Zunvest = (log(prccf/Xunvest)+  matunvest*(rfunvest-bs_yield+sigmasq/2))/(sigma*sqrt(matunvest));

if matunvest=. or matunvest<0 then Zunvest=.;
if opts_unvested_val_excl_curryear=. or opts_unvested_val_excl_curryear<0 
or opts_unvested_num_excl_curryear<0 or opts_unvested_num_excl_curryear=. then Zunvest=.;

if opts_unvested_val_excl_curryear=. or opts_unvested_val_excl_curryear<0 
or opts_unvested_num_excl_curryear<0 or opts_unvested_num_excl_curryear=. then FZunvest=.;

Sunvest = exp(-bs_yield* matunvest)*probnorm(Zunvest)*(prccf/100)*opts_unvested_num_excl_curryear;

if Zunvest^=. then do;
Runvest = exp(-bs_yield* matunvest)*PDF('normal',Zunvest,0,1)*prccf*sqrt( matunvest)*0.01*opts_unvested_num_excl_curryear;
end;
if Zunvest=. then Runvest=.;

Vunvest = opts_unvested_num_excl_curryear*(prccf*exp(-bs_yield* matunvest)*probnorm(Zunvest)
     -Xunvest*exp(-rfunvest* matunvest)*probnorm(Zunvest-sigma*sqrt( matunvest)));
if opts_unvested_num_excl_curryear=. or Zunvest=. then Vunvest=.;

matvest=matunvest-3;
if matvest<0 and matvest^=. then matvest=0.001;

Xvest=prccf-(opts_vested_val_excl_curryear/opts_vested_num_excl_curryear);
if (Xvest<0 and Xvest^=.) then Xvest=0.01;

if round(matvest,1) <=1 then rfvest=oneyr;
if round(matvest,1) =2  then rfvest=twoyr;
if round(matvest,1) =3  then rfvest=threeyr;
if round(matvest,1) =4  then rfvest=fouryr;
if round(matvest,1) =5  then rfvest=fiveyr;
if round(matvest,1) =6  then rfvest=sixyr;
if round(matvest,1) =7  then rfvest=sevenyr;
if round(matvest,1) =8  then rfvest=eightyr;
if round(matvest,1) =9  then rfvest=nineyr;
if round(matvest,1)=10  then rfvest=tenyr;

if (matvest^=. and round(matvest,10)>10) then rfvest=tenyr;

Zvest = (log(prccf/Xvest)+  matvest*(rfvest-bs_yield+sigmasq/2))/(sigma*sqrt(matvest));
if matvest=. or matvest<0 then Zvest=.;
if opts_vested_val_excl_curryear=. or opts_vested_val_excl_curryear<0 
or opts_vested_num_excl_curryear<0 or opts_vested_num_excl_curryear=. 
then Zvest=.;


Svest = exp(-bs_yield* matvest)*probnorm(Zvest)*(prccf/100)*opts_vested_num_excl_curryear;

if Zvest^=. then do;
Rvest = exp(-bs_yield* matvest)*PDF('normal',Zvest,0,1)*prccf*sqrt( matvest)*0.01*opts_vested_num_excl_curryear;
end;
if Zvest=. then Rvest=.;

Vvest = opts_vested_num_excl_curryear*(prccf*exp(-bs_yield*matvest)*probnorm(Zvest)
     -Xvest*exp(-rfvest*matvest)*probnorm(Zvest-sigma*sqrt(matvest)));
if opts_vested_num_excl_curryear=. or Zvest=. then Vvest=.;

Sshr= shrown*prccf/100;

Vshr = shrown*prccf;

if opts_vested_num_excl_curryear=0 then do;
Svest=0;
Rvest=0;
Vvest=0;
end;
if opts_unvested_num_excl_curryear=0 then do;
Sunvest=0;
Runvest=0;
Vunvest=0;
end;


delta = sum(Svest, Sunvest, Sopts_grants_yearend, Sshr);
optiondelta = sum(Svest, Sunvest, Sopts_grants_yearend);
sharedelta = Sshr;

Ropt = sum(Rvest, Runvest, Ropts_grants_yearend);

Vportfolio_yearend = sum(Vvest, Vunvest, Vopts_yearend, Vshr);

if old_datafmt_flag=0 then do;
Svest=.; Sunvest=.; Sopts_grants_yearend=.; Sshr=.;
Ropt=.; Rvest=.;  Runvest=.; Ropts_grants_yearend=.;
Vportfolio_yearend=.; Vvest=.;  Vunvest=.; Vopts_yearend=.; Vshr=.;
delta=.; 	 
end;

run;

data excomp15;
set excomp14;
keep coperol year delta optiondelta sharedelta ropt Vportfolio_yearend exec_fullname;
run;
proc sql;
create table excomp16 as 
select *
from excomp15 x left join anncomp y
on x.coperol=y.coperol and x.year=y.year;

data outstandingawards;
set execcomp.outstandingawards;
coperol=co_per_rol;
opts_vested_num = opts_unex_exer;
opts_unvested_num = opts_unex_unexer;
opts_unearned_num = opts_unex_unearn ;
expric = round(expric,0.01);
keep coperol year eip_shrs_unvest_num eip_shrs_unvest_val expric
opts_vested_num opts_unvested_num opts_unearned_num outawdnum
shrs_unvest_num shrs_unvest_val exdate;
proc sort nodupkey;
by coperol year outawdnum;

proc means data=outstandingawards noprint n mean p1 p25 p50 p75 p99;
var opts_unearned_num;
where opts_unearned_num>0 and opts_unearned_num^=.;
run;

data excomp13_subset;
set excomp13;
keep  coperol year shrown bs_yield fyenddt
sigmasq sigma prccf  year 
oneyr twoyr threeyr fouryr fiveyr sixyr sevenyr eightyr nineyr tenyr;
proc sort nodupkey;
by coperol year;
run;

proc sql;
create table outstandingawards1 as
select *
from outstandingawards  x left join excomp13_subset y
on x.coperol=y.coperol and x.year=y.year;

data outstandingawards1a; 
set outstandingawards1;

maturity_yearend = (exdate-fyenddt)/365;

if round(maturity_yearend,1) =1  then rfyearend=oneyr;
if round(maturity_yearend,1) =2  then rfyearend=twoyr;
if round(maturity_yearend,1) =3  then rfyearend=threeyr;
if round(maturity_yearend,1) =4  then rfyearend=fouryr;
if round(maturity_yearend,1) =5  then rfyearend=fiveyr;
if round(maturity_yearend,1) =6  then rfyearend=sixyr;
if round(maturity_yearend,1) =7  then rfyearend=sevenyr;
if round(maturity_yearend,1) =8  then rfyearend=eightyr;
if round(maturity_yearend,1) =9  then rfyearend=nineyr;
if round(maturity_yearend,1)=10  then rfyearend=tenyr;

if (maturity_yearend^=. and round(maturity_yearend,1)>10) then rfyearend=tenyr;

Xc = expric;


Zc_yearend= (log(prccf/Xc)+ maturity_yearend*(rfyearend-bs_yield+sigmasq/2))/(sigma*sqrt( maturity_yearend));
if maturity_yearend=. then Zc_yearend=.;


Sunvest_yearend= exp(-bs_yield*maturity_yearend)*probnorm(Zc_yearend)*opts_unvested_num*prccf/100;
if Zc_yearend=. then Sunvest_yearend=.;
if opts_unvested_num=. or opts_unvested_num=0 then Sunvest_yearend=0;

Svest_yearend= exp(-bs_yield*maturity_yearend)*probnorm(Zc_yearend)*opts_vested_num*prccf/100;
if Zc_yearend=. then Svest_yearend=.;
if opts_vested_num=. or opts_vested_num=0 then Svest_yearend=0;

Vunvest_yearend= opts_unvested_num*(prccf*exp(-bs_yield* maturity_yearend)*probnorm(Zc_yearend)
-Xc*exp(-rfyearend*maturity_yearend)*probnorm(Zc_yearend-sigma*sqrt( maturity_yearend)));
if Zc_yearend=. then Vunvest_yearend=.;
if opts_unvested_num=. or opts_unvested_num=0 then Vunvest_yearend=0;

Vvest_yearend= opts_vested_num*(prccf*exp(-bs_yield* maturity_yearend)*probnorm(Zc_yearend)
-Xc*exp(-rfyearend*maturity_yearend)*probnorm(Zc_yearend-sigma*sqrt( maturity_yearend)));
if Zc_yearend=. then Vvest_yearend=.;
if opts_vested_num=. or opts_vested_num=0 then Vvest_yearend=0;

if Zc_yearend^=. then do;
Runvest_yearend = exp(-bs_yield* maturity_yearend)*PDF('normal',Zc_yearend,0,1)*prccf*sqrt( maturity_yearend)*0.01*opts_unvested_num;
end;
if  Zc_yearend=. or maturity_yearend=. then Runvest_yearend=.;
if opts_unvested_num=. or opts_unvested_num=0 then Runvest_yearend=0;

if Zc_yearend^=. then do;
Rvest_yearend = exp(-bs_yield* maturity_yearend)*PDF('normal',Zc_yearend,0,1)*prccf*sqrt( maturity_yearend)*0.01*opts_vested_num;
end;
if  Zc_yearend=. or maturity_yearend=. then Rvest_yearend=.;
if opts_vested_num=. or opts_vested_num=0 then Rvest_yearend=0;

drop oneyr twoyr threeyr fouryr fiveyr sixyr
sevenyr eightyr nineyr tenyr ;
run;

proc sql;
create table outstandingawards2
as select *, 
sum(opts_unvested_num) as sumunvested_num_opts,
sum(opts_vested_num) as sumvested_num_opts,
sum(Sunvest_yearend) as sumSunvest_yearend,
sum(Svest_yearend) as sumSvest_yearend,
sum(Runvest_yearend) as sumRunvest_yearend,
sum(Rvest_yearend) as sumRvest_yearend,
sum(Vunvest_yearend) as sumVunvest_yearend,
sum(Vvest_yearend) as sumVvest_yearend
from outstandingawards1a
group by coperol,year;


data outstandingawards3;
set outstandingawards2;
if sumunvested_num_opts=0 or sumunvested_num_opts=. then do;
sumSunvest_yearend=0; 
sumRunvest_yearend=0; 
sumVunvest_yearend=0; 
end;
if sumvested_num_opts=0 then do;
sumSvest_yearend=0; 
sumRvest_yearend=0; 
sumVvest_yearend=0; 
end;
newRopt = sum(sumRvest_yearend, sumRunvest_yearend);
Sshr= shrown*prccf/100;
newVshr= shrown*prccf;
newVportfolio_yearend = sum(sumVvest_yearend, sumVunvest_yearend, newVshr);
* delta;
newdelta = sum(sumSvest_yearend, sumSunvest_yearend, Sshr);
newoptiondelta = sum(sumSvest_yearend, sumSunvest_yearend);
newsharedelta = Sshr;
drop opts_vested_num opts_unvested_num shrown;
proc sort nodupkey ;
by coperol year;
run;
proc sql; 
create table excomp17 as
select *
from excomp16 x left join outstandingawards3 y
on x.coperol=y.coperol and x.year=y.year;

data excomp18;
set excomp17;
if old_datafmt_flag=0 then do;
delta=newdelta;
optiondelta=newoptiondelta;
sharedelta=newsharedelta;
Ropt = newRopt;
Vportfolio_yearend = newVportfolio_yearend;
end;
firm_related_wealth=Vportfolio_yearend;
vega=ropt;
keep coperol year delta gvkey;
proc sort nodupkey;
by coperol year;
proc sort data=excomp18; by year coperol; run;

data delta;
set excomp18;
keep gvkey coperol year delta;
run;
proc download data=delta out=result.delta; run;

endrsubmit;
signoff;
