/*******************************************************************

**This file contains the program used to create the following datasets:
1. the main EPS sample used to conduct the forecast-level tests in Table 2;
2. the GAAP sample used to control for the majority accounting basis in Table 2;
3. the consensus sample used to conduct the firm-quarter level tests in Tables 2 and 4;
4. the datasets with additional variables for cross-sectional tests in Tables 3, 5 and 6.


*******************************************************************/

*PATH;
LIBNAME RAW 'D:\IBES\RAW';
LIBNAME RESULT "D:\IBES\RESULT";

/******************************************************************************

1. the main EPS sample used to conduct the forecast-level tests in Table 2;

*******************************************************************************/

/******************************************************************************

-Step 1.1: Get the most recent forecast for q issued by each brokerage house before EA_q;

*******************************************************************************/

*GET ALL QUARTERLY EARNINGS (EPS) ESTIMATES;
DATA DET_1 
(KEEP = TICKER ANALYS FPEDATS EST ANNDATS REVDATS FPI ESTIMATOR ANNTIMS ACTDATS);
SET RAW.IBES_DETAIL_QTR;
EST = VALUE;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
IF TICKER NE "";
IF ANALYS NE .;
IF EST NE .;
RUN;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS DESCENDING ANNTIMS; QUIT;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS; QUIT;

*GET ALL ACTUAL EARNINGS (EPS) ANNOUNCEMENTS;
DATA ACT_0;
SET RAW.ACTUAL;
FPEDATS = PENDS;
IF PDICITY = 'QTR';
IF MEASURE = 'EPS';
IF VALUE NE .;
IF ANNDATS NE .;
FORMAT FPEDATS YYMMDDN8.;
RUN;
PROC SORT DATA = ACT_0 NODUPKEY; BY TICKER FPEDATS; QUIT;

/*MERGE IN PERMNO*/
DATA RET; SET RAW.DAILY_RET;
KEEP NCUSIP PERMNO PRC DATE;
RUN;
PROC SORT DATA = RET NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE ACT_PERMNO AS
SELECT A.TICKER, A.ANNDATS, B.PERMNO, B.DATE AS CRSP_DATE, B.PRC
FROM ACT_0 A LEFT JOIN RET B
ON A.CUSIP = B.NCUSIP AND 0 <= A.ANNDATS - B.DATE <= 4;
QUIT;
DATA ACT_PERMNO_0; SET ACT_PERMNO; IF NOT MISSING(PERMNO); RUN;
PROC SORT DATA = ACT_PERMNO_0; BY TICKER ANNDATS CRSP_DATE; QUIT;
PROC SORT DATA = ACT_PERMNO_0 OUT = ACT_PERMNO_1 NODUPKEY; BY TICKER ANNDATS; QUIT;

PROC SQL;
CREATE TABLE ACT_1 AS
SELECT A.*, B.PERMNO, B.PRC AS EA_PRC
FROM ACT_0 A LEFT JOIN ACT_PERMNO_1 B
ON A.TICKER = B.TICKER AND A.ANNDATS = B.ANNDATS;
QUIT;

*MERGE IN LAST QTR'S EARNINGS (EPS) ANNOUNCEMENT;
PROC SQL;
CREATE TABLE ACT_2 AS
SELECT A.TICKER, A.CUSIP, A.PERMNO, A.USFIRM, 
A.VALUE AS CURR_ACT, A.FPEDATS AS CURR_FPE, A.ANNDATS AS CURR_ANN, 
B.VALUE AS PREV_ACT, B.FPEDATS AS PREV_FPE, B.EA_PRC AS PRC, B.ANNDATS AS PREV_ANN
FROM ACT_1 AS A, ACT_1 AS B
WHERE A.TICKER = B.TICKER 
AND  0 < A.FPEDATS - B.FPEDATS <= 110
AND  A.ANNDATS > B.ANNDATS;
QUIT; 
PROC SORT DATA = ACT_2; BY TICKER CURR_FPE CURR_ANN DESCENDING PREV_FPE;QUIT;
PROC SORT DATA = ACT_2 OUT = ACT_21 NODUPKEY; BY TICKER CURR_FPE CURR_ANN;QUIT;

*MERGE ANALYST FORECASTS WITH ACTUAL EPS;
PROC SQL;
CREATE TABLE DET_2 AS
SELECT A.*, 
B.ESTIMATOR, B.ANALYS, B.EST AS CURR_EST_FIN, 
B.ANNDATS AS CURR_EST_FIN_ANN, B.REVDATS, B.ACTDATS
FROM ACT_21 AS A, DET_1 AS B
WHERE A.TICKER = B.TICKER 
AND A.CURR_FPE = B.FPEDATS
AND A.CURR_ANN > B.ANNDATS;
QUIT;
*KEEP THE MOST RECENT FORECAST ISSUED BY EACH BROKERAGE HOUSE BEFORE EA;
PROC SORT DATA = DET_2; BY TICKER CURR_FPE CURR_ANN ESTIMATOR DESCENDING CURR_EST_FIN_ANN; QUIT;
PROC SORT DATA = DET_2 OUT = DET_21 NODUPKEY; BY TICKER CURR_FPE CURR_ANN ESTIMATOR; QUIT;

/*******************************************************************************************************************/
/*******************************************************************************************************************/
/*******************************************************************************************************************/

/******************************************************************************

-Step 1.2: Identify whether the forecast is removed and whether the analyst stopped issued forecasts for the firm;

*******************************************************************************/
*IDENTIFY REMOVED ESTIMATES;
DATA EXCLUDED; SET RAW.EXCLUDED; 
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
RUN;

PROC SQL;
CREATE TABLE DET_3 AS
SELECT A.*, B.EXCFLA = 'X' AS FL_EXCL, B.EXCDATS, B.ACTDATS AS ACTDATS_EXCL, B.VALUE AS VALUE_EXCL
FROM DET_21 A LEFT JOIN EXCLUDED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS 
AND A.CURR_FPE = B.FPEDATS AND B.EXCENDS GT A.CURR_ANN AND A.ACTDATS = B.ACTDATS AND A.CURR_EST_FIN = B.VALUE;
QUIT;

*IDENTIFY STOPPED ESTIMATES;
PROC SQL;
CREATE TABLE DET_4 AS
SELECT A.*, B.ASTPDATS NE . AS FL_STOP, B.ASTPDATS
FROM DET_3 A LEFT JOIN  RAW.STOPPED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.CURR_FPE = B.FPEDATS AND B.ASTPDATS LE A.CURR_ANN
AND A.CURR_EST_FIN_ANN LE B.ASTPDATS AND B.PDICITY='Q' AND B.MEASURE = "EPS";
QUIT; 

PROC SORT DATA = DET_4 NODUPKEY; BY TICKER CURR_FPE CURR_ANN ESTIMATOR DESCENDING FL_EXCL DESCENDING FL_STOP DESCENDING ASTPDATS; QUIT;
PROC SORT DATA = DET_4 NODUPKEY; BY TICKER CURR_FPE CURR_ANN ESTIMATOR; QUIT; 

/*ELIMINATE FORECASTS ISSUED BY ANALYSTS THAT HAVE STOPPED BEFORE THE FORECASTS WERE REMOVED*/
DATA DET_5; SET DET_4;
IF CURR_ACT = . THEN DELETE;
IF EXCDATS = . THEN DO; IF ASTPDATS NE . THEN DELETE;END;
IF EXCDATS NE . THEN DO;   
	IF ASTPDATS NE . THEN DO;
		IF ASTPDATS LE EXCDATS THEN DELETE;END;END;
IF EXCDATS GE CURR_ANN THEN FL_EXCL = 0;
IF CURR_EST_FIN_ANN GE CURR_ANN THEN DELETE;
RUN;

/*******************************************************************************************************************/
/*******************************************************************************************************************/
/*******************************************************************************************************************/

/******************************************************************************

-Step 1.3: Create test variables;

*******************************************************************************/


/*ADJUST STOCK SPLITS*/
DATA RET_CFA; SET RAW.DAILY_RET;
KEEP PERMNO PRC CFACSHR DATE;
RUN;
PROC SORT DATA = RET_CFA NODUPREC; BY _ALL_; QUIT;

PROC SORT DATA = DET_5 (KEEP = TICKER PERMNO CURR_FPE CURR_ANN) OUT = ACTUAL_ADJ0 NODUPKEY;
BY  TICKER PERMNO CURR_FPE CURR_ANN;
QUIT;

PROC SQL;
CREATE TABLE ACTUAL_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_FPE, A.CURR_ANN, 
B.CFACSHR, B.DATE AS CRSP_DATE
FROM ACTUAL_ADJ0 A LEFT JOIN RET_CFA B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_ANN -  B.DATE >= 0;
QUIT;
PROC SORT DATA = ACTUAL_ADJ; BY TICKER PERMNO CURR_FPE CURR_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = ACTUAL_ADJ OUT = ACTUAL_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_FPE CURR_ANN; QUIT;

PROC SQL;
CREATE TABLE DET_51 AS
SELECT A.*, B.CFACSHR AS ACT_ADJ
FROM DET_5 A LEFT JOIN ACTUAL_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.CURR_ANN = B.CURR_ANN;
QUIT;

PROC SQL;
CREATE TABLE DET_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM DET_51 A LEFT JOIN RET_CFA B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_EST_FIN_ANN -  B.DATE >= 0;
QUIT;

PROC SORT DATA = DET_ADJ; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = DET_ADJ OUT = DET_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN;QUIT;

PROC SQL;
CREATE TABLE DET_52 AS
SELECT A.*, B.CFACSHR AS DET_ADJ
FROM DET_51 A LEFT JOIN DET_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND
A.CURR_EST_FIN_ANN = B.CURR_EST_FIN_ANN;
QUIT;

/*DETERMINANTS OF FORECAST REMOVAL*/
DATA SE_1; SET DET_52;
IF MISSING(ACT_ADJ) = 0 AND MISSING(DET_ADJ) = 0 AND (ACT_ADJ NE DET_ADJ) THEN DO;
	CURR_EST_FIN = CURR_EST_FIN*(ACT_ADJ/DET_ADJ);
END;
IF MISSING(PREV_ANN) = 0;
IF MISSING(CURR_ANN) = 0;
IF MISSING(REVDATS) = 0;
IF PRC <= 0 THEN DELETE;
IF REVDATS > CURR_ANN OR REVDATS = CURR_ANN THEN REVDATS = CURR_EST_FIN_ANN;
ABS_FE = ABS(CURR_ACT - CURR_EST_FIN)/PRC;
DUMMY180 =  (CURR_ANN - CURR_EST_FIN_ANN) > 180;
DUMMY105 =  (CURR_ANN - REVDATS) > 105;
PRE_PREVANN = REVDATS < PREV_ANN;
HORIZON_REV = REVDATS - PREV_ANN; 
HORIZON_FORE_REV = REVDATS - CURR_EST_FIN_ANN;
RUN;
/*Note: HORIZON_REV and HORIZON_FORE_REV are transformed to percentile ranks that range from 0 to 1  in STATA for regression analysis.*/

*IDENTIFY 4TH QUARTER;
DATA FOURTHQTR; SET RAW.ACTUAL;
IF PDICITY = 'ANN';
IF MEASURE = 'EPS';
KEEP TICKER FPEDATS;
RUN;

PROC SQL;
CREATE TABLE SE_11 AS
SELECT A.*, B.FPEDATS AS FOURTH_FPEDATS
FROM SE_1 A LEFT JOIN FOURTHQTR B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.FPEDATS;
QUIT;

DATA SE_12; SET SE_11;
IF CURR_FPE  = FOURTH_FPEDATS THEN DO;
DUMMY105 =  (CURR_ANN - REVDATS) > 120;END;
/*For the 4th-quarter forecasts, IBES uses 120 days instead of 105 days to identify stale forecasts.*/
RUN;

/*Optimism: Compare with other analyst forecasts*/
DATA SE_2; SET SE_12;
IF EXCDATS NE . AND EXCDATS < CURR_ANN THEN OPT_DATE = EXCDATS;
IF EXCDATS NE . AND EXCDATS GE CURR_ANN THEN OPT_DATE = CURR_ANN -1;
IF EXCDATS = . THEN OPT_DATE = CURR_ANN-1;
FORMAT OPT_DATE YYMMDD.;
RUN;

PROC SQL;
CREATE TABLE OPT_OTHER AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.OPT_DATE,
B.ESTIMATOR AS OTHER_ESTIMATOR, B.ANALYS AS OTHER_ANALYS, 
B.CURR_EST_FIN_ANN AS OTHER_ANNDATS, B.CURR_EST_FIN AS OTHER_EST
FROM SE_2 A LEFT JOIN RESULT.SE_190516 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.ESTIMATOR NE B.ESTIMATOR AND A.ANALYS NE B.ANALYS
AND (B.FL_EXCL = 0 OR B.EXCDATS = . OR B.EXCDATS > A.OPT_DATE)
AND (B.ASTPDATS = . OR B.ASTPDATS > A.OPT_DATE) 
AND A.OPT_DATE GE B.CURR_EST_FIN_ANN;
QUIT;
/*Note: RESULT.SE_190516 is generated by Additional_Input.sas */

PROC SQL;
CREATE TABLE OPT_OTHERLATEST AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.OPT_DATE, A.OTHER_ESTIMATOR, A.OTHER_ANALYS, MAX(A.OTHER_ANNDATS) AS OTHER_ANNDATS
FROM OPT_OTHER A
GROUP BY A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.OPT_DATE, A.OTHER_ESTIMATOR, A.OTHER_ANALYS;
QUIT;

PROC SQL;
CREATE TABLE OPT_OTHERLATEST2 AS
SELECT A.*, B.OTHER_EST
FROM OPT_OTHERLATEST A LEFT JOIN OPT_OTHER B
ON A.TICKER=B.TICKER AND A.CURR_FPE=B.CURR_FPE AND 
A.ESTIMATOR=B.ESTIMATOR AND A.ANALYS=B.ANALYS AND 
A.CURR_EST_FIN_ANN=B.CURR_EST_FIN_ANN AND A.OPT_DATE=B.OPT_DATE AND 
A.OTHER_ESTIMATOR=B.OTHER_ESTIMATOR AND A.OTHER_ANALYS=B.OTHER_ANALYS AND 
A.OTHER_ANNDATS=B.OTHER_ANNDATS;
QUIT;

PROC SQL;
CREATE TABLE OPT_OTHER2 AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.OPT_DATE,
MEDIAN(A.OTHER_EST) AS MEDIAN_EST
FROM OPT_OTHERLATEST2 A
GROUP BY A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.OPT_DATE;
QUIT;

PROC SQL;
CREATE TABLE SE_3 AS
SELECT A.*, B.MEDIAN_EST
FROM SE_2 A LEFT JOIN OPT_OTHER2 B
ON A.TICKER=B.TICKER AND A.CURR_FPE=B.CURR_FPE
AND A.ESTIMATOR=B.ESTIMATOR AND A.ANALYS=B.ANALYS
AND A.CURR_EST_FIN_ANN=B.CURR_EST_FIN_ANN
AND A.OPT_DATE=B.OPT_DATE;
QUIT;

DATA SE_4; SET SE_3;
OPT_MEDIANOTHER = CURR_EST_FIN > MEDIAN_EST;
RUN;

/*SPLIT OPTIMISM*/
PROC SQL;
CREATE TABLE CSS AS 
SELECT A.TICKER, A.CURR_FPE, SUM(A.CURR_EST_FIN) AS SUM_EST, COUNT(A.ESTIMATOR) AS ALL_EST 
FROM SE_4 A
GROUP BY A.TICKER, A.CURR_FPE;

CREATE TABLE SE_5 AS
SELECT A.*, B.SUM_EST, B.ALL_EST
FROM SE_4  A LEFT JOIN CSS B
ON A.TICKER=B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

DATA SE_6; SET SE_5; 
AFT_EXCL = (SUM_EST-CURR_EST_FIN)/(ALL_EST-1);
AFT_EXCL = ROUND(AFT_EXCL,0.01);
PRE_EXCL = SUM_EST/ALL_EST;
PRE_EXCL = ROUND(PRE_EXCL,0.01);
IF CURR_ACT >= AFT_EXCL THEN DO;
IF CURR_ACT < PRE_EXCL THEN FAIL_MTB = 1; 
END;
IF FAIL_MTB NE 1 THEN FAIL_MTB = 0;

IF OPT_MEDIANOTHER = 1 AND FAIL_MTB = 1 THEN OPT_MTB = 1;
IF OPT_MTB NE 1 THEN OPT_MTB = 0;
IF OPT_MEDIANOTHER = 1 AND FAIL_MTB = 0 THEN OPT_NOT_MTB = 1;
IF OPT_NOT_MTB NE 1 THEN OPT_NOT_MTB = 0;
RUN;

/*IDENTIFY THE MANAGEMENT FORECAST FOR EACH ANALYST FORECAST*/
DATA MGMT_BASE; SET RAW.MGMT_GUIDE;
IF MEASURE = 'EPS';
IF PDICITY = 'QTR';
IF VAL_1 NE . THEN RVAL1 = ROUND(VAL_1, 0.01);
IF VAL_2 NE . THEN RVAL2 = ROUND(VAL_2, 0.01);
IF RVAL1 NE . AND RVAL2 NE . THEN DO; MIN_VAL = RVAL1; MAX_VAL = RVAL2;END;
IF RVAL1 NE . AND RVAL2 = . THEN DO;
IF RANGE_DESC IN ('03','04','08','11','12','16') THEN DO; MIN_VAL = RVAL1; MAX_VAL = 999999;END;
IF RANGE_DESC IN ('06','10','13','15','17')  THEN DO; MAX_VAL = RVAL1; MIN_VAL=-999999;END;
IF RANGE_DESC IN ('02','05','07','09','14') THEN POINT = RVAL1;
END;
RUN;

PROC SQL;
CREATE TABLE MGMT_BASE_0 AS
SELECT A.TICKER, A.CURR_FPE, A.PREV_ANN, A.CURR_ANN, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.CURR_EST_FIN, 
B.MIN_VAL, B.MAX_VAL, B.POINT, B.ANNDATS AS MGMT_ANNDATS,
 (B.ANNDATS - A.CURR_EST_FIN_ANN) AS DAYS_BTW
FROM SE_6 A LEFT JOIN MGMT_BASE B
ON A.TICKER=B.TICKER AND YEAR(A.CURR_FPE) = B.PRD_YR AND MONTH(A.CURR_FPE) = B.PRD_MON AND A.CURR_EST_FIN_ANN < B.ANNDATS;
QUIT;

DATA MGMT_BASE_1; SET MGMT_BASE_0; IF NOT MISSING(DAYS_BTW); RUN;

PROC SORT DATA = MGMT_BASE_1; BY TICKER CURR_FPE CURR_ANN ESTIMATOR ANALYS CURR_EST_FIN_ANN CURR_EST_FIN DAYS_BTW;QUIT;
PROC SORT DATA = MGMT_BASE_1 OUT = MGMT_BASE_2 NODUPKEY; BY TICKER CURR_FPE CURR_ANN ESTIMATOR ANALYS CURR_EST_FIN_ANN CURR_EST_FIN;QUIT;

DATA MGMT_BASE_3; SET MGMT_BASE_2;
IF CURR_EST_FIN NE . THEN RCURR_EST_FIN = ROUND(CURR_EST_FIN, 0.01);
/*                                             DEFINE "OUTSIDE THE RANGE"                                                                                 */
/*DETAIL ESTIMATES WHICH HAVE NOT BEEN UPDATED OR CONFIRMED FOLLOWING THE ISSUANCE OF GUIDANCE */
/*AND DO NOT FALL WITHIN THE GUIDANCE RANGE (E.G. $1.00 - $1.10) WILL BE FILTERED / EXCLUDED FROM THE MEAN AT THE TIME OF GUIDANCE. */
/*IN THOSE CASES WHERE SINGLE-POINT GUIDANCE IS ISSUED (E.G. ABOUT $1.00), ESTIMATES NOT WITHIN 5% OF THE GUIDANCE WILL BE FOOTNOTED AND EXCLUDED FROM THE MEAN. */
IF MISSING(MIN_VAL) = 0 AND MISSING(MAX_VAL) = 0 AND MISSING(POINT) = 1 THEN DO; 
IF RCURR_EST_FIN < MIN_VAL OR MAX_VAL < RCURR_EST_FIN THEN OUT_RANGE = 1;
END; 
IF MISSING(POINT) = 0 THEN DO; 
IF RCURR_EST_FIN < 0.95*POINT OR 1.05*POINT < RCURR_EST_FIN THEN OUT_RANGE = 1;
END;
IF OUT_RANGE NE 1 THEN OUT_RANGE = 0;
RUN;

PROC SQL;
CREATE TABLE SE_7 AS
SELECT A.*, (B.OUT_RANGE = 1) AS OUT_RANGE
FROM SE_6 A LEFT JOIN MGMT_BASE_3 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.CURR_ANN = B.CURR_ANN AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND A.CURR_EST_FIN_ANN = B.CURR_EST_FIN_ANN AND A.CURR_EST_FIN = B.CURR_EST_FIN;
QUIT;

*DROP OBSERVATIONS BEFORE YEAR 1994;
DATA SE_8; SET SE_7;
IF YEAR(CURR_ANN) < 1994 THEN DELETE;
IF (PREV_ANN <= REVDATS & REVDATS <= PREV_ANN+10) OR (PREV_ANN <= CURR_EST_FIN_ANN & CURR_EST_FIN_ANN <= PREV_ANN+10) THEN REV10 = 1;
IF REV10 NE 1 THEN REV10 = 0;
RUN;

/*******************************************************************************************************************/
/*******************************************************************************************************************/
/*******************************************************************************************************************/

/******************************************************************************

-Step 1.4: Export the sample to stata for regression analysis;

*******************************************************************************/
/*Note: Continuous variables are winsorized in STATA*/
DATA RESULT.MAINSAMPLE_EPS; SET SE_8; RUN;
PROC EXPORT DATA = RESULT.MAINSAMPLE_EPS
OUTFILE = "D:\IBES\result_202008\table2_eps.dta"
DBMS = STATA REPLACE;
QUIT;

/*******************************************************************************************************************/
/*******************************************************************************************************************/
/*******************************************************************************************************************/

/******************************************************************************

2. the GAAP sample used to control for the majority accounting basis in Table 2;

*******************************************************************************/

/******************************************************************************

-Step 2.1: Get quarterly GAAP EPS forecasts;

*******************************************************************************/

*GET ALL QUARTERLY GAAP EARNINGS ESTIMATES;
DATA GAAPEPS_1 
(KEEP = TICKER MEASURE ANALYS FPEDATS EST ANNDATS REVDATS FPI ESTIMATOR ANNTIMS ACTDATS);
SET RAW.DETAIL_IBES_ALL; 
EST = VALUE;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'GPS';
IF TICKER NE "";
IF ANALYS NE .;
IF EST NE .;
RUN;
PROC SORT DATA = GAAPEPS_1 NODUPKEY; BY TICKER FPEDATS ANALYS ANNDATS DESCENDING ANNTIMS; QUIT;
PROC SORT DATA = GAAPEPS_1 OUT = GAAPEPS_2 NODUPKEY; BY TICKER FPEDATS ANALYS ANNDATS;QUIT;

*GET ALL ACTUAL GAAP EARNINGS ANNOUNCEMENTS;
DATA GAAPACT_1; SET RAW.ACTUAL_ALL;
IF PDICITY = 'QTR';
IF MEASURE = 'GPS';
IF VALUE NE .;
IF ANNDATS NE .;
FPEDATS = PENDS;
FORMAT FPEDATS YYMMDDN8.;
RUN;
PROC SORT DATA=GAAPACT_1; BY TICKER FPEDATS DESCENDING ANNDATS; QUIT;
PROC SORT DATA = GAAPACT_1 OUT = GAAPACT_2 NODUPKEY; BY TICKER FPEDATS; QUIT;

*CREATE A LIST OF ALL CONSECUTIVE ACTUALS;
PROC SQL;
CREATE TABLE GAAPACT_3 AS
SELECT 
A.TICKER, A.CUSIP, A.MEASURE, A.USFIRM, A.VALUE AS CURR_GAAPACT, A.FPEDATS AS CURR_GAAPFPE, A.ANNDATS AS CURR_GAAPANN,  
B.VALUE AS PREV_GAAPACT, B.FPEDATS AS PREV_GAAPFPE, B.ANNDATS AS PREV_GAAPANN
FROM GAAPACT_2 AS A, GAAPACT_2 AS B
WHERE A.TICKER = B.TICKER AND 
A.FPEDATS BETWEEN B.FPEDATS + 80 AND B.FPEDATS + 100;
QUIT; 
PROC SORT DATA = GAAPACT_3 NODUPKEY; BY TICKER CURR_GAAPFPE; QUIT;

*MERGE IN ANALYST GAAP FORECASTS;
PROC SQL;
CREATE TABLE GAAPEPS_3 AS
SELECT A.*, 
B.ESTIMATOR, B.ANALYS, B.EST AS CURR_EST_GAAP, B.ANNDATS AS CURR_EST_GAAP_ANN, 
B.REVDATS AS GAAP_REV, B.ACTDATS AS GAAP_ACTDATS
FROM GAAPEPS_2 B LEFT JOIN GAAPACT_3 A
ON A.TICKER = B.TICKER AND A.CURR_GAAPFPE = B.FPEDATS AND A.CURR_GAAPANN > B.ANNDATS;
QUIT;

*IDENTIFY REMOVED GAAP ESTIMATES;
DATA EXCLUDED; SET RAW.EXCLUDED_ALL;
IF MEASURE = 'GPS';
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
RUN;

PROC SQL;
CREATE TABLE GAAPEPS_4 AS
SELECT A.*, B.EXCFLA = 'X' AS FL_GAAPEXCL, B.EXCDATS AS GAAP_EXCDATS, 
B.ACTDATS AS ACTDATS_GAAPEXCL, B.VALUE AS VALUE_GAAPEXCL
FROM GAAPEPS_3 A LEFT JOIN EXCLUDED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS 
AND A.CURR_GAAPFPE = B.FPEDATS AND B.EXCENDS GT A.CURR_GAAPANN AND A.GAAP_ACTDATS = B.ACTDATS AND A.CURR_EST_GAAP = B.VALUE;
QUIT;

*IDENTIFY STOPPED GAAP ESTIMATES;
DATA STOPPED; SET RAW.STOPPED_ALL;
IF PDICITY = 'Q';
IF MEASURE = 'GPS';
RUN;

PROC SQL;
CREATE TABLE GAAPEPS_5 AS
SELECT A.*, B.ASTPDATS NE . AS FL_GAAPSTOP, B.ASTPDATS AS GAAP_ASTPDATS
FROM GAAPEPS_4 A LEFT JOIN STOPPED AS B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.CURR_GAAPFPE = B.FPEDATS
AND B.ASTPDATS LE A.CURR_GAAPANN AND A.CURR_EST_GAAP_ANN LE B.ASTPDATS AND B.PDICITY = "Q";
QUIT;
PROC SORT DATA = GAAPEPS_5; BY TICKER CURR_GAAPFPE ESTIMATOR ANALYS CURR_EST_GAAP_ANN CURR_EST_GAAP
DESCENDING FL_GAAPSTOP DESCENDING GAAP_ASTPDATS; QUIT;
PROC SORT DATA = GAAPEPS_5 OUT = GAAPEPS_6 NODUPKEY; 
BY TICKER CURR_GAAPFPE ESTIMATOR ANALYS CURR_EST_GAAP_ANN CURR_EST_GAAP; QUIT; 

DATA GAAPEPS_7; SET GAAPEPS_6;
IF CURR_EST_GAAP_ANN GE CURR_GAAPANN THEN DELETE;
IF GAAP_ASTPDATS NE . THEN DELETE;
IF GAAP_EXCDATS GE CURR_GAAPANN THEN FL_GAAPEXCL = 0;
RUN;

/*ADJUST STOCK SPLITS FOR GAAP SAMPLE*/
DATA RET; SET RAW.DAILY_RET;
KEEP PERMNO CFACSHR DATE;
RUN;
PROC SORT DATA = RET NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE GAAPEPS_9 AS
SELECT A.*, B.PERMNO
FROM GAAPEPS_7 A LEFT JOIN ACT_1 B
ON A.TICKER = B.TICKER AND A.CURR_GAAPFPE = B.FPEDATS;
QUIT;

PROC SORT DATA = GAAPEPS_9 (KEEP = TICKER PERMNO CURR_GAAPFPE CURR_GAAPANN) OUT = GACTUAL_ADJ0 NODUPKEY;
BY  TICKER PERMNO CURR_GAAPFPE CURR_GAAPANN;
QUIT;

PROC SQL;
CREATE TABLE GACTUAL_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_GAAPFPE, A.CURR_GAAPANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM GACTUAL_ADJ0 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_GAAPANN -  B.DATE >= 0;
QUIT;
PROC SORT DATA = GACTUAL_ADJ; BY TICKER PERMNO CURR_GAAPFPE CURR_GAAPANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = GACTUAL_ADJ OUT = GACTUAL_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_GAAPFPE CURR_GAAPANN; QUIT;

PROC SQL;
CREATE TABLE GAAPEPS_91 AS
SELECT A.*, B.CFACSHR AS ACT_ADJ
FROM GAAPEPS_9 A LEFT JOIN GACTUAL_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_GAAPFPE = B.CURR_GAAPFPE AND A.CURR_GAAPANN = B.CURR_GAAPANN;
QUIT;

PROC SQL;
CREATE TABLE GDET_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_GAAPFPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_GAAP_ANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM GAAPEPS_91 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_EST_GAAP_ANN -  B.DATE >= 0;
QUIT;

PROC SORT DATA = GDET_ADJ; BY TICKER PERMNO CURR_GAAPFPE ESTIMATOR ANALYS CURR_EST_GAAP_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = GDET_ADJ OUT = GDET_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_GAAPFPE ESTIMATOR ANALYS CURR_EST_GAAP_ANN;QUIT;

PROC SQL;
CREATE TABLE GAAPEPS_92 AS
SELECT A.*, B.CFACSHR AS DET_ADJ
FROM GAAPEPS_91 A LEFT JOIN GDET_ADJ1 B
ON A.TICKER = B.TICKER AND A.CURR_GAAPFPE = B.CURR_GAAPFPE AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND
A.CURR_EST_GAAP_ANN = B.CURR_EST_GAAP_ANN;
QUIT;

DATA SE_GAAP; SET GAAPEPS_92;
IF MISSING(ACT_ADJ) = 0 AND MISSING(DET_ADJ) = 0 AND (ACT_ADJ NE DET_ADJ) THEN DO;
	CURR_EST_GAAP = CURR_EST_GAAP*(ACT_ADJ/DET_ADJ);
END;
RUN;

/*********************************************************************************************************/
/*********************************************************************************************************/
/*********************************************************************************************************/

/******************************************************************************

-Step 2.2: Identify those EPS forecasts that are issued by the same analyst on the same day and are equal to the GAAP forecasts;

*******************************************************************************/

PROC SQL;
CREATE TABLE EPS_FORE AS
SELECT A.*, B.CURR_EST_GAAP
FROM RESULT.MAINSAMPLE_EPS A LEFT JOIN SE_GAAP B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_GAAPFPE AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS
AND A.CURR_EST_FIN_ANN = B.CURR_EST_GAAP_ANN AND A.CURR_EST_FIN = B.CURR_EST_GAAP;
QUIT;

DATA EPS_FORE1; SET EPS_FORE;
IF CURR_EST_GAAP NE .;
RUN;

/*********************************************************************************************************/
/*********************************************************************************************************/
/*********************************************************************************************************/

/******************************************************************************

-Step 2.3: Export the GAAP sample to STATA for regression analysis

*******************************************************************************/
/*Note: Continuous variables are winsorized in STATA*/
DATA RESULT.MAINSAMPLE_GAAP; SET EPS_FORE1; RUN;
PROC EXPORT DATA=EPS_FORE1
OUTFILE='D:\IBES\result_202008\table2_gaap.dta' 
DBMS=STATA REPLACE; 
RUN;

/*********************************************************************************************************/
/*********************************************************************************************************/
/*********************************************************************************************************/

/******************************************************************************

3. the consensus sample used to conduct the firm-quarter level tests in Tables 2 and 4;

*******************************************************************************/

/******************************************************************************

-Step 3.1: Get IBES consensus from IBES summary file;

*******************************************************************************/
/*READ IN IBES CONSENSUS FROM SUMMARY FILE*/
DATA SUMMARY; SET RAW.SUMMARY_QTR_UNADJUSTED; 
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
RUN;

PROC SQL;
CREATE TABLE SUMMARY0 AS
SELECT A.TICKER, A.FPEDATS, MAX(A.STATPERS) AS STATPERS
FROM SUMMARY A
GROUP BY A.TICKER, A.FPEDATS;
QUIT;

PROC SQL;
CREATE TABLE SUMMARY1 AS
SELECT A.*
FROM SUMMARY A, SUMMARY0 B
WHERE A.TICKER = B.TICKER AND A.FPEDATS = B.FPEDATS AND A.STATPERS = B.STATPERS;
QUIT;

/*ADD ACTUAL EPS TO THE CONSENSUS FILE*/
PROC SQL;
CREATE TABLE SUMMARY2 AS
SELECT A.*, B.CURR_ACT AS ACTUAL, B.PERMNO, B.PRC, B.CURR_ANN, B.PREV_ANN
FROM SUMMARY1 A LEFT JOIN ACT_21 B
ON A.TICKER = B.TICKER AND A.FPEDATS = B.CURR_FPE;
QUIT;

/*ADJUST STOCK SPLITS FOR THE SUMMARY FILE*/
PROC SQL;
CREATE TABLE SUMMARY_ADJ AS
SELECT A.TICKER, A.PERMNO, A.FPEDATS, A.STATPERS, B.CFACSHR, B.DATE AS CRSP_DATE
FROM SUMMARY2 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.STATPERS -  B.DATE >= 0;
QUIT;
PROC SORT DATA = SUMMARY_ADJ; BY TICKER FPEDATS STATPERS DESCENDING CRSP_DATE; QUIT;
PROC SORT DATA = SUMMARY_ADJ OUT = SUMMARY_ADJ1 NODUPKEY; BY TICKER FPEDATS STATPERS; QUIT;

PROC SQL;
CREATE TABLE SUMMARY20 AS
SELECT A.*, B.CFACSHR AS SUM_ADJ
FROM SUMMARY2 A LEFT JOIN SUMMARY_ADJ1 B 
ON A.TICKER = B.TICKER AND A.FPEDATS = B.FPEDATS AND A.STATPERS = B.STATPERS;
QUIT;

PROC SQL;
CREATE TABLE ACTUAL_ADJ AS
SELECT A.TICKER, A.PERMNO, A.FPEDATS, A.CURR_ANN, B.CFACSHR, B.DATE AS CRSP_DATE
FROM SUMMARY20 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.CURR_ANN -  B.DATE >= 0;
QUIT;
PROC SORT DATA = ACTUAL_ADJ; BY TICKER PERMNO FPEDATS CURR_ANN DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = ACTUAL_ADJ OUT = ACTUAL_ADJ1 NODUPKEY; BY TICKER PERMNO FPEDATS CURR_ANN; QUIT;

PROC SQL;
CREATE TABLE SUMMARY21 AS
SELECT A.*, B.CFACSHR AS ACT_ADJ
FROM SUMMARY20 A LEFT JOIN ACTUAL_ADJ1 B
ON A.TICKER = B.TICKER AND A.FPEDATS = B.FPEDATS AND A.CURR_ANN = B.CURR_ANN;
QUIT;

/*If adjustment factors on STATPERS and CURR_ANN are not the same,
adjust the MEANEST in the summary file to be on the same basis with actual EPS at q*/
DATA SUMMARY22; SET SUMMARY21;
IF MISSING(ACT_ADJ) = 0 AND MISSING(SUM_ADJ) = 0 AND (ACT_ADJ NE SUM_ADJ) THEN DO;
	MEANEST = (ACT_ADJ/SUM_ADJ)*MEANEST;
END;
RUN;

/*BACK OUT THE CONSENSUS BEFORE REMOVALS*/
DATA DET_11; SET RESULT.MAINSAMPLE_EPS;
IF DUMMY180 = 1 OR DUMMY105 = 1 OR PRE_PREVANN = 1 OR OUT_RANGE = 1 THEN STALE = 1; 
IF STALE NE 1 THEN STALE = 0;
IF STALE = 0 THEN FRESH = 1;
IF STALE = 1 THEN FRESH = 0;
RUN;
PROC SORT DATA = DET_11 NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE FINAL_EXCLUSION1 AS
SELECT A.TICKER, A.CURR_FPE, 
SUM(A.CURR_EST_FIN*A.FL_EXCL) AS SUM_EXCLU, SUM(A.FL_EXCL) AS N_EXCLU,
SUM(A.CURR_EST_FIN*A.STALE*A.FL_EXCL) AS SUM_EXCLUPOLICY1, SUM(A.FL_EXCL*A.STALE)  AS N_EXCLUPOLICY1,                                
SUM(A.CURR_EST_FIN*A.FRESH*A.FL_EXCL) AS SUM_EXCLUDISCRETION1, SUM(A.FL_EXCL*A.FRESH) AS N_EXCLUDISCRETION1
FROM DET_11 A
GROUP BY A.TICKER, A.CURR_FPE;
QUIT;
PROC SORT DATA = FINAL_EXCLUSION1 NODUPRECS; BY _ALL_ ; RUN;

/*********************************************************************************************************/
/*********************************************************************************************************/
/*********************************************************************************************************/

/******************************************************************************

-Step 3.2: Create test variables for the consensus sample;

*******************************************************************************/

PROC SQL;
CREATE TABLE SUMMARY3 AS
SELECT A.*, 
B.SUM_EXCLU, B.N_EXCLU, 
B.SUM_EXCLUPOLICY1, B.N_EXCLUPOLICY1,
B.SUM_EXCLUDISCRETION1, B.N_EXCLUDISCRETION1
FROM SUMMARY22 A LEFT JOIN FINAL_EXCLUSION1 B
ON A.TICKER = B.TICKER AND A.FPEDATS=B.CURR_FPE;
QUIT;
PROC SORT DATA = SUMMARY3 NODUPREC; BY _ALL_; QUIT;

DATA SUMMARY4; SET SUMMARY3;
IF ACTUAL = . THEN DELETE;
IF SUM_EXCLU = . THEN SUM_EXCLU = 0;
IF N_EXCLU = . THEN N_EXCLU = 0;
IF SUM_EXCLUPOLICY1 = . THEN SUM_EXCLUPOLICY1 = 0;
IF SUM_EXCLUDISCRETION1 = . THEN SUM_EXCLUDISCRETION1 = 0;
IF N_EXCLUPOLICY1 = . THEN N_EXCLUPOLICY1 = 0;
IF N_EXCLUDISCRETION1 = . THEN N_EXCLUDISCRETION1 = 0;

/*MAIN X VARIABLES*/
/*These variables are winsorized in STATA before the regression analysis.*/
IF N_EXCLU > 0 THEN EXCLU_SUM = (SUM_EXCLU/N_EXCLU - MEANEST)/PRC;
IF N_EXCLU = 0 THEN EXCLU_SUM = 0;
IF N_EXCLUPOLICY1 > 0 THEN EXCLU_POLICY = (SUM_EXCLUPOLICY1/N_EXCLUPOLICY1 - MEANEST)/PRC;
IF N_EXCLUPOLICY1 = 0 THEN EXCLU_POLICY = 0;
IF N_EXCLUDISCRETION1 > 0 THEN EXCLU_DISCRETION = (SUM_EXCLUDISCRETION1/N_EXCLUDISCRETION1 - MEANEST)/PRC;
IF N_EXCLUDISCRETION1 = 0 THEN EXCLU_DISCRETION = 0;
RUN;

DATA SUMMARY7; SET SUMMARY4;
YEAR_QTR = YEAR(FPEDATS)*10+QTR(FPEDATS); 
CONSENSUS_BF = (MEANEST*NUMEST + SUM_EXCLU)/(NUMEST+N_EXCLU);
CSS_BF_DISCRETION = (MEANEST*NUMEST + SUM_EXCLUDISCRETION1)/(NUMEST+N_EXCLUDISCRETION1);
IF QTR(FPEDATS) = 1 THEN LAG1QTR = (YEAR(FPEDATS) - 1)*10 + 4;
IF QTR(FPEDATS) > 1 THEN LAG1QTR = YEAR(FPEDATS)*10 + QTR(FPEDATS) - 1;
RUN;

/*MERGE IN FIRM CHARACTERISTICS*/
DATA COMP0; SET RAW.COMP_QTRFIRMCHAR;
IF ATQ = 0 THEN DELETE;
IF ATQ = . THEN DELETE;
IF ATQ < 0 THEN DELETE;
IF FQTR = 1 THEN LAG1QTR = (FYEARQ-1)*10+4;
IF FQTR > 1 THEN LAG1QTR = FYEARQ*10 + FQTR - 1;
RUN;
PROC SORT DATA=COMP0 OUT = COMP1 NODUPKEY; BY GVKEY FYEARQ FQTR; RUN;

PROC SQL;
CREATE TABLE COMP2 AS
SELECT A.*, B.ATQ AS LAG1QTR_ATQ
FROM COMP1 AS A, COMP1 AS B
WHERE A.GVKEY = B.GVKEY AND A.LAG1QTR = B.FYEARQ*10 + B.FQTR;
QUIT;
PROC SORT DATA = COMP2 NODUPREC; BY _ALL_;QUIT;
DATA BTM_1;SET COMP2;
MVE = LOG(CSHOQ*PRCCQ);
BOOK = CEQQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN BOOK = SEQQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN BOOK = ATQ - LTQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN DELETE;
OP = SUM(REVTQ,-COGSQ,-XSGAQ,-XINTQ,-XRDQ)/BOOK; /*PROFITABILITY*/
CMA = (ATQ - LAG1QTR_ATQ)/LAG1QTR_ATQ; /*ASSET GROWTH*/
IF ATQ = . THEN DELETE;
IF ATQ LE 0 THEN DELETE;
BTM = BOOK/(CSHOQ*PRCCQ);
RUN;

PROC SQL;
CREATE TABLE BTM_2 AS
SELECT A.*, B.LPERMNO AS PERMNO
FROM BTM_1 AS A LEFT JOIN RAW.CCMXPF_LNKHIST AS B
ON A.GVKEY = B.GVKEY AND 
B.LINKTYPE IN ("LC", "LS", "LU") AND
(B.LINKDT <= A.DATADATE OR MISSING(B.LINKDT) = 1) AND (A.DATADATE <= B.LINKENDDT OR B.LINKENDDT = .E);               
QUIT;
PROC SORT DATA = BTM_2 NODUPKEY OUT = BTM_3;BY GVKEY PERMNO FYEARQ FQTR; RUN;

PROC SQL;
CREATE TABLE BTM_4 AS
SELECT A.PERMNO, A.FPEDATS, B.GVKEY, B.BOOK, B.DATADATE, B.OP, B.CMA, B.BTM, B.MVE, B.SIC, B.CSHOQ
FROM SUMMARY7 A LEFT JOIN BTM_3 B
ON A.PERMNO = B.PERMNO AND A.FPEDATS > B.DATADATE;
QUIT;
PROC SORT DATA = BTM_4; BY PERMNO FPEDATS DESCENDING DATADATE;QUIT;
PROC SORT DATA = BTM_4 NODUPKEY; BY PERMNO FPEDATS; QUIT;

PROC SQL;
CREATE TABLE SUMMARY71 AS
SELECT A.*, B.GVKEY, B.BOOK, B.DATADATE, B.OP, B.CMA, B.BTM, B.MVE, B.SIC, B.CSHOQ
FROM SUMMARY7 A LEFT JOIN BTM_4 B
ON A.PERMNO = B.PERMNO AND A.FPEDATS = B.FPEDATS;
QUIT;

/*FORECAST DISPERSION; NUMBER OF ANALYST FOLLOWING*/
PROC SQL;
CREATE TABLE VOL_END AS
SELECT A.TICKER, A.CURR_FPE, STD(A.CURR_EST_FIN) AS VOL_END, COUNT(A.CURR_EST_FIN) AS NUM_END
FROM DET_11 A
GROUP BY A.TICKER, A.CURR_FPE;
QUIT;

PROC SQL;
CREATE TABLE SUMMARY710 AS
SELECT A.*, B.VOL_END, B.NUM_END
FROM SUMMARY71 A LEFT JOIN VOL_END B
ON A.TICKER = B.TICKER AND A.FPEDATS = B.CURR_FPE;
QUIT;

DATA SUMMARY711; SET SUMMARY710;
IF ACTUAL = . THEN DELETE;
YEAR_QTR = YEAR(FPEDATS)*10+QTR(FPEDATS);
IF PRC <= 0 THEN DELETE;
IF MVE = . THEN MVE = LOG(CSHOQ*PRC);
IF BTM = . THEN BTM = (CSHOQ*PRC)/BOOK;
LOSS = (ACTUAL < 0);
FORE_DISPERSION = LOG(VOL_END/PRC+1);
FOLLOWING = LOG(NUM_END+1);
RUN;

/*MEET OR BEAT & CONSENSUS ERRORS*/
DATA SUMMARY72; SET SUMMARY711; 
MTB_CONSENBF = ACTUAL - CONSENSUS_BF;
MTB_CONSENBF = ROUND(MTB_CONSENBF, 0.01);

IF MTB_CONSENBF >=0 THEN MTB_CONSENBF_DUM = 1;
IF MTB_CONSENBF_DUM ~= 1 THEN MTB_CONSENBF_DUM = 0;

IF MTB_CONSENBF >=0 AND MTB_CONSENBF <=0.01 THEN MTB_JUST1_CONSENBF = 1;
IF MTB_JUST1_CONSENBF ~= 1 THEN MTB_JUST1_CONSENBF = 0;
LABEL MTB_JUST1_CONSENBF = "$0 <= (ACTUAL - CONSENSUS BF. REMOVAL) <= $0.01";
ABSFE_CONSENBF = ABS(ACTUAL - CONSENSUS_BF)/PRC;

MTB_CSS_BFDISCRETION = ACTUAL - CSS_BF_DISCRETION;
MTB_CSS_BFDISCRETION = ROUND(MTB_CSS_BFDISCRETION, 0.01);

IF MTB_CSS_BFDISCRETION >=0 THEN MTB_CSSBFDISC_DUM = 1;
IF MTB_CSSBFDISC_DUM ~= 1 THEN MTB_CSSBFDISC_DUM = 0;

IF MTB_CSS_BFDISCRETION >=0 AND MTB_CSS_BFDISCRETION <=0.01 THEN MTB_JUST1_CSSBFDISC = 1;
IF MTB_JUST1_CSSBFDISC ~= 1 THEN MTB_JUST1_CSSBFDISC = 0;
LABEL MTB_JUST1_CSSBFDISC = "$0 <= (ACTUAL - CONSENSUS BF. DISCRETIONARY REMOVAL) <= $0.01";
ABSFE_CSSBFDISC = ABS(ACTUAL - CSS_BF_DISCRETION)/PRC;

MTB_IBES = ACTUAL - MEANEST;
MTB_IBES = ROUND(MTB_IBES, 0.01);

IF MTB_IBES >= 0 THEN MTB_IBES_DUM = 1;
IF MTB_IBES_DUM ~= 1 THEN MTB_IBES_DUM = 0;

IF MTB_IBES >= 0 AND MTB_IBES <= 0.01 THEN MTB_JUST1_IBES = 1;
IF MTB_JUST1_IBES ~= 1 THEN MTB_JUST1_IBES = 0;
LABEL MTB_JUST1_IBES = "ROUNDED-$0 <= (ACTUAL - IBES_MEAN_CONSENSUS) <= $0.01";
ABSFE_IBES = ABS(ACTUAL - MEANEST)/PRC;

LABEL ABSFE_CONSENBF = "ABSFE_CONSENSUS BF. REMOVAL";
LABEL ABSFE_CSSBFDISC = "ABSFE_CONSENSUS BF. DISCRETIONARY REMOVAL";
LABEL ABSFE_IBES = "ABSFE_IBES CONSENSUS";

CSS_ERR_CHG =   ABSFE_IBES  - ABSFE_CONSENBF;
RUN;

/*********************************************************************************************************/
/*********************************************************************************************************/
/*********************************************************************************************************/

/******************************************************************************

-Step 3.3: Eliminate firm-qtrs with at least one downward mgmt guidance;

*******************************************************************************/
DATA MGMT 
(KEEP = TICKER ANNDATS MEASURE PDICITY GUIDANCE_CODE PRD_MON PRD_YR MGMT_DUMMY DOWN_DUMMY); 
SET RAW.MGMT_GUIDE;
MGMT_DUMMY = 1;
IF MEASURE = "EPS";
IF MISSING(VAL_1) = 0 OR MISSING(VAL_2) = 0;
IF PDICITY = "QTR";
IF GUIDANCE_CODE = "01" THEN DOWN_DUMMY = 1; ELSE DOWN_DUMMY = 0;
RUN;

/*ADD FPEDATS, EA_Q AND EA_Q-1 TO THE GUIDANCE DATASET*/
PROC SORT DATA = RESULT.MAINSAMPLE_EPS (KEEP = TICKER CURR_FPE CURR_ANN PREV_ANN) OUT = PREV_ANN NODUPKEY;
BY TICKER CURR_FPE CURR_ANN PREV_ANN;
QUIT;

PROC SQL;
CREATE TABLE MGMT_1 AS
SELECT A.*, B.CURR_ANN, B.PREV_ANN, B.CURR_FPE
FROM MGMT A LEFT JOIN PREV_ANN B
ON A.TICKER = B.TICKER AND A.PRD_MON = MONTH(B.CURR_FPE) AND A.PRD_YR = YEAR(B.CURR_FPE);
QUIT;

/*KEEP THOSE GUIDANCE ISSUED AFTER EA_Q-1*/
DATA MGMT_2; SET MGMT_1;
IF ANNDATS > PREV_ANN + 5;
IF ANNDATS < CURR_ANN;
IF CURR_FPE NE .;
RUN;

PROC SQL;
CREATE TABLE MGMT_3 AS
SELECT A.TICKER, A.CURR_FPE, MAX(A.DOWN_DUMMY) AS DOWN_DUMMY
FROM MGMT_2 A
GROUP BY A.TICKER, A.CURR_FPE;
QUIT;

PROC SQL;
CREATE TABLE SUMMARY73 AS
SELECT A.*, B.DOWN_DUMMY
FROM SUMMARY72 A LEFT JOIN MGMT_3 B
ON A.TICKER = B.TICKER AND A.FPEDATS = B.CURR_FPE;
QUIT;

DATA SUMMARY74; SET SUMMARY73;
IF DOWN_DUMMY = . THEN DOWN_DUMMY = 0;
IF DOWN_DUMMY = 1 THEN DELETE;
RUN;

/*******************************************************************************************************************/
/*******************************************************************************************************************/
/*******************************************************************************************************************/

/******************************************************************************

-Step 3.4: Export the sample to stata for regression analysis;

*******************************************************************************/
/*Note: Continuous variables are winsorized in STATA*/
DATA RESULT.MAINSAMPLE_CSS; SET SUMMARY74; RUN;
PROC EXPORT 
DATA= RESULT.MAINSAMPLE_CSS
FILE='D:\IBES\RESULT_202008\TABLE24_FIRMQTR.DTA' 
DBMS=STATA REPLACE; 
RUN;

/******************************************************************************

4. the datasets with additional variables for cross-sectional tests in Tables 3, 5 and 6.

*******************************************************************************/

*Inclusion in the S&P 500 index;
DATA CROSS_BASE_0; SET RESULT.MAINSAMPLE_EPS;
CUSIP6 = SUBSTR(CUSIP,1,6);
RUN;

DATA INDEX_BASE; SET RAW.SPIDX_CST;
IF INDEXID = "500";
CUSIP6 = SUBSTR(CUSIPX, 1, 6);
RUN;
PROC SORT DATA = INDEX_BASE (KEEP = CUSIP6 CONMX INDEXID DATADATE) OUT = INDEX_BASE1 NODUPKEY;
BY CUSIP6 INDEXID DATADATE;
QUIT;

PROC SQL;
CREATE TABLE INDEX_BASE2 AS
SELECT A.CUSIP6, A.CURR_FPE, A.PREV_ANN, B.INDEXID, B.DATADATE AS INDEX_DATADATE
FROM CROSS_BASE_0 A LEFT JOIN INDEX_BASE B
ON A.CUSIP6 = B.CUSIP6 AND 0<= A.PREV_ANN - B.DATADATE <=120;
QUIT;
PROC SORT DATA = INDEX_BASE2; BY CUSIP6 CURR_FPE PREV_ANN DESCENDING INDEX_DATADATE; QUIT;
PROC SORT DATA = INDEX_BASE2 OUT = INDEX_BASE3 NODUPKEY; BY CUSIP6 CURR_FPE PREV_ANN; QUIT;

PROC SQL;
CREATE TABLE CROSS_BASE_1 AS
SELECT A.*, B.INDEXID
FROM CROSS_BASE_0 A LEFT JOIN INDEX_BASE3 B
ON A.CUSIP6 = B.CUSIP6 AND A.CURR_FPE = B.CURR_FPE AND A.PREV_ANN = B.PREV_ANN;
QUIT;

DATA CROSS_BASE_2; SET CROSS_BASE_1;
IF MISSING(INDEXID) = 1 THEN SP500 = 0;
IF MISSING(INDEXID) = 0 THEN SP500 = 1;
RUN;

PROC EXPORT DATA= CROSS_BASE_2
OUTFILE='D:\IBES\RESULT_202008\CROSS_SP500.DTA' 
DBMS=STATA REPLACE; 
RUN;

*Coverage;
PROC SQL;
CREATE TABLE BASE_COVERAGE AS
SELECT A.TICKER, A.CURR_FPE, COUNT(DISTINCT A.ESTIMATOR) AS COVERAGE
FROM CROSS_BASE_2 A
GROUP BY A.TICKER, A.CURR_FPE;
QUIT;

PROC SQL;
CREATE TABLE CROSS_BASE_3 AS
SELECT A.*, B.COVERAGE 
FROM CROSS_BASE_2 A LEFT JOIN BASE_COVERAGE B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

/*Note: the variable COVERAGE is converted into a dummy variable in STATA before the regression analysis

EGEN COVERAGE_MEDIAN=XTILE(COVERAGE), N(2)
REPLACE COVERAGE_MEDIAN = COVERAGE_MEDIAN - 1

*/

PROC EXPORT DATA= CROSS_BASE_3
OUTFILE='D:\IBES\RESULT_202008\CROSS_COVERAGE.DTA' 
DBMS=STATA REPLACE; 
RUN;

*Firm Size;
PROC SORT DATA = RAW.MONTH_CRSP  (KEEP = PERMNO DATE PRC SHROUT) OUT = SIZE; BY PERMNO DATE PRC SHROUT; QUIT;

PROC SQL;
CREATE TABLE BASE_SIZE AS
SELECT A.PERMNO, A.CURR_ANN, B.SHROUT, B.DATE
FROM CROSS_BASE_3 A LEFT JOIN SIZE B
ON A.PERMNO = B.PERMNO AND 0 <= A.PREV_ANN - B.DATE <= 120;
QUIT;
PROC SORT DATA = BASE_SIZE; BY PERMNO CURR_ANN DESCENDING DATE; QUIT;
PROC SORT DATA = BASE_SIZE OUT = BASE_SIZE1 NODUPKEY; BY PERMNO CURR_ANN; QUIT;

PROC SQL;
CREATE TABLE CROSS_BASE_4 AS
SELECT A.*, B.SHROUT
FROM CROSS_BASE_3 A LEFT JOIN BASE_SIZE1 B
ON A.PERMNO = B.PERMNO AND A.CURR_ANN = B.CURR_ANN;
QUIT;

DATA CROSS_BASE_5; SET CROSS_BASE_4;
SIZE = PRC*SHROUT;
RUN;

/*Note: the variable SIZE is converted into a dummy variable in STATA before the regression analysis

EGEN SIZE_MEDIAN=XTILE(SIZE), N(2)
REPLACE SIZE_MEDIAN = SIZE_MEDIAN - 1

*/

PROC EXPORT DATA= CROSS_BASE_5
OUTFILE='D:\IBES\RESULT_202008\CROSS_SIZE.DTA' 
DBMS=STATA REPLACE; 
RUN;

/*******************************************************************************************/
*Long vs. Short;
DATA LONG_1 
(KEEP = TICKER ANALYS FPEDATS EST ANNDATS REVDATS FPI ESTIMATOR ANNTIMS ACTDATS);
SET RAW.IBES_DETAIL_QTR;
EST = VALUE;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
IF TICKER NE "";
IF ANALYS NE .;
IF EST NE .;
RUN;
PROC SORT DATA = LONG_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS DESCENDING ANNTIMS; QUIT;
PROC SORT DATA = LONG_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS; QUIT;

*IDENTIFY EXCLUDED ESTIMATES;
DATA EXCLUDED; SET RAW.EXCLUDED;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
RUN;

PROC SQL;
CREATE TABLE LONG_2 AS
SELECT A.*, B.EXCFLA = 'X' AS FL_EXCL, B.EXCDATS, B.ACTDATS AS ACTDATS_EXCL, B.VALUE AS VALUE_EXCL, B.EXCENDS
FROM LONG_1  A LEFT JOIN EXCLUDED B
ON A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS 
AND A.FPEDATS = B.FPEDATS AND A.ACTDATS = B.ACTDATS AND A.EST = B.VALUE;
QUIT;

PROC SQL;
CREATE TABLE LONG_3 AS
SELECT A.TICKER, A.CURR_FPE, A.CURR_ANN, A.PREV_ANN, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN,A.PRC, A.FOURTH_FPEDATS,
B.FPEDATS AS COUNTER_CURRFPE,
B.EST AS LONG_EST, B.FL_EXCL AS LONG_FLEXCL, B.EXCDATS AS LONG_EXCDATS, B.EXCENDS AS LONG_EXCENDS, B.REVDATS AS LONG_REVDATS
FROM RESULT.MAINSAMPLE_EPS A LEFT JOIN LONG_2 B
ON A.TICKER = B.TICKER AND A.CURR_FPE < B.FPEDATS AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND
A.CURR_EST_FIN_ANN = B.ANNDATS;
QUIT;
PROC SORT DATA = LONG_3 OUT = LONG_4 NODUPRECS; BY _ALL_ ; QUIT;

DATA LONG_5; SET LONG_4;
IF MISSING(COUNTER_CURRFPE) = 1 THEN DELETE;
IF LONG_EXCDATS GE CURR_ANN THEN LONG_FLEXCL = 0;
IF LONG_EXCDATS NE . AND LONG_EXCENDS < CURR_ANN THEN LONG_FLEXCL = 0;

IF LONG_REVDATS > CURR_ANN OR LONG_REVDATS = CURR_ANN THEN LONG_REVDATS = CURR_EST_FIN_ANN;

LONG_DUMMY180 =  (CURR_ANN - CURR_EST_FIN_ANN) > 180;
LONG_DUMMY105 =  (CURR_ANN - LONG_REVDATS) > 105;
IF FOURTH_FPEDATS = 1 THEN LONG_DUMMY105 =  (CURR_ANN - LONG_REVDATS) > 120;
LONG_PRE_PREVANN = LONG_REVDATS < PREV_ANN;
LONG_HORIZON_REV = LONG_REVDATS - PREV_ANN; 
LONG_HORIZON_FORE_REV = LONG_REVDATS - CURR_EST_FIN_ANN;
RUN;

PROC SQL;
CREATE TABLE LONG_6 AS
SELECT A.*, B.VALUE AS LONG_ACTUAL
FROM LONG_5 A LEFT JOIN ACT_1 B
ON A.TICKER=B.TICKER AND A.COUNTER_CURRFPE = B.FPEDATS;
QUIT;
DATA LONG_7; SET LONG_6;
LONG_ABS_FE = ABS(LONG_ACTUAL - LONG_EST)/PRC;
IF LONG_EXCDATS NE . AND LONG_EXCDATS < CURR_ANN THEN OPT_DATE = LONG_EXCDATS;
IF LONG_EXCDATS NE . AND LONG_EXCDATS GE CURR_ANN THEN OPT_DATE = CURR_ANN -1;
IF LONG_EXCDATS = . THEN OPT_DATE = CURR_ANN-1;
FORMAT OPT_DATE YYMMDD.;
RUN;

PROC SQL;
CREATE TABLE LONG_OPT AS
SELECT A.TICKER, A.CURR_FPE, A.CURR_ANN, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE,
B.ESTIMATOR AS OTHER_ESTIMATOR, B.ANALYS AS OTHER_ANALYS, 
B.ANNDATS AS OTHER_ANNDATS, B.EST AS OTHER_EST, B.EXCDATS AS LONG_EXCDATS
FROM LONG_7 A LEFT JOIN LONG_2 B
ON A.TICKER = B.TICKER AND A.COUNTER_CURRFPE = B.FPEDATS AND A.ESTIMATOR NE B.ESTIMATOR AND A.ANALYS NE B.ANALYS
AND (B.EXCDATS = . OR B.EXCDATS > A.OPT_DATE)
AND A.OPT_DATE GE B.ANNDATS;
QUIT;

PROC SQL;
CREATE TABLE STOPPED AS
SELECT A.TICKER, A.COUNTER_CURRFPE, A.OTHER_ESTIMATOR, 
B.ASTPDATS AS LONG_ASTPDATS
FROM LONG_OPT A, RAW.STOPPED B
WHERE A.TICKER = B.TICKER AND A.ESTIMATOR = B.ESTIMATOR AND A.COUNTER_CURRFPE = B.FPEDATS AND 
B.ASTPDATS LE A.CURR_ANN
AND A.CURR_EST_FIN_ANN LE B.ASTPDATS AND B.PDICITY='Q' AND B.MEASURE = "EPS";
QUIT; 
PROC SORT DATA = STOPPED NODUPRECS; BY _ALL_ ; QUIT;
PROC SORT DATA = STOPPED (KEEP = TICKER COUNTER_CURRFPE OTHER_ESTIMATOR) OUT = STOPPED1 NODUPKEY; 
BY TICKER COUNTER_CURRFPE OTHER_ESTIMATOR; QUIT;

PROC SQL;
CREATE TABLE LONG_OPT1 AS
SELECT A.*, (B.OTHER_ESTIMATOR NE . ) AS STOPPED
FROM LONG_OPT A LEFT JOIN STOPPED1 B
ON A.TICKER=B.TICKER AND A.COUNTER_CURRFPE=B.COUNTER_CURRFPE AND A.OTHER_ESTIMATOR=B.OTHER_ESTIMATOR;
QUIT;
DATA LONG_OPT2; SET LONG_OPT1;
IF STOPPED = 1 THEN DELETE;
RUN;

PROC SQL;
CREATE TABLE LONG_OTHERLATEST AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE,
A.OTHER_ESTIMATOR, A.OTHER_ANALYS, MAX(A.OTHER_ANNDATS) AS OTHER_ANNDATS
FROM LONG_OPT2 A
GROUP BY A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE,
A.OTHER_ESTIMATOR, A.OTHER_ANALYS;
QUIT;

PROC SQL;
CREATE TABLE LONG_OTHERLATEST2 AS
SELECT A.*, B.OTHER_EST
FROM LONG_OTHERLATEST A LEFT JOIN LONG_OPT2 B
ON A.TICKER=B.TICKER AND A.CURR_FPE=B.CURR_FPE AND 
A.ESTIMATOR=B.ESTIMATOR AND A.ANALYS=B.ANALYS AND 
A.CURR_EST_FIN_ANN=B.CURR_EST_FIN_ANN AND A.COUNTER_CURRFPE = B.COUNTER_CURRFPE AND
A.OTHER_ESTIMATOR=B.OTHER_ESTIMATOR AND A.OTHER_ANALYS=B.OTHER_ANALYS AND 
A.OTHER_ANNDATS=B.OTHER_ANNDATS;
QUIT;

PROC SQL;
CREATE TABLE LONG_OPT3 AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE,
MEDIAN(A.OTHER_EST) AS OPT_BENCHMARK,
SUM(A.OTHER_EST) AS SUM_OTHER,
COUNT(A.OTHER_ANALYS) AS N_OTHER
FROM LONG_OTHERLATEST2 A
GROUP BY A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE;
QUIT;

PROC SQL;
CREATE TABLE LONG_8 AS
SELECT A.*, 
B.OPT_BENCHMARK, B.SUM_OTHER, B.N_OTHER
FROM LONG_7 A LEFT JOIN LONG_OPT3 B
ON A.TICKER=B.TICKER AND A.CURR_FPE=B.CURR_FPE AND 
A.ESTIMATOR=B.ESTIMATOR AND A.ANALYS=B.ANALYS AND 
A.CURR_EST_FIN_ANN=B.CURR_EST_FIN_ANN AND A.COUNTER_CURRFPE = B.COUNTER_CURRFPE;
QUIT;

DATA LONG_9; SET LONG_8;
IF LONG_ABS_FE = . THEN DELETE;
IF MISSING(OPT_BENCHMARK) = 1 THEN LONG_OPT_MEDIANOTHER = 0;
IF MISSING(OPT_BENCHMARK) = 0 THEN DO;
IF LONG_EST > OPT_BENCHMARK THEN LONG_OPT_MEDIANOTHER = 1;
IF LONG_EST <= OPT_BENCHMARK THEN LONG_OPT_MEDIANOTHER = 0;END;
IF YEAR(CURR_ANN) < 1994 THEN DELETE; /*to be consistent with the sample period of the main sample*/

AFT_EXCL = (SUM_OTHER)/(N_OTHER);
AFT_EXCL = ROUND(AFT_EXCL,0.01);
PRE_EXCL = (SUM_OTHER+LONG_EST)/(N_OTHER+1);
PRE_EXCL = ROUND(PRE_EXCL,0.01);
IF LONG_ACTUAL >= AFT_EXCL THEN DO;
IF LONG_ACTUAL < PRE_EXCL THEN FAIL_MTB = 1; 
END;
IF FAIL_MTB NE 1 THEN FAIL_MTB = 0;

IF LONG_OPT_MEDIANOTHER = 1 AND FAIL_MTB = 1 THEN LONG_OPT_MTB = 1;
IF LONG_OPT_MTB NE 1 THEN LONG_OPT_MTB = 0;
IF LONG_OPT_MEDIANOTHER = 1 AND FAIL_MTB = 0 THEN LONG_OPT_NMTB = 1;
IF LONG_OPT_NMTB NE 1 THEN LONG_OPT_NMTB = 0;
RUN;

/*IDENTIFY THE MANAGEMENT FORECAST FOR EACH ANALYST FORECAST*/
DATA MGMT_BASE; SET RAW.MGMT_GUIDE;
IF MEASURE = 'EPS';
IF PDICITY = 'QTR';
IF VAL_1 NE . THEN RVAL1 = ROUND(VAL_1, 0.01);
IF VAL_2 NE . THEN RVAL2 = ROUND(VAL_2, 0.01);
IF RVAL1 NE . AND RVAL2 NE . THEN DO; MIN_VAL = RVAL1; MAX_VAL = RVAL2;END;
IF RVAL1 NE . AND RVAL2 = . THEN DO;
IF RANGE_DESC IN ('03','04','08','11','12','16') THEN DO; MIN_VAL = RVAL1; MAX_VAL = 999999;END;
IF RANGE_DESC IN ('06','10','13','15','17')  THEN DO; MAX_VAL = RVAL1; MIN_VAL=-999999;END;
IF RANGE_DESC IN ('02','05','07','09','14') THEN POINT = RVAL1;
END;
RUN;

PROC SQL;
CREATE TABLE MGMT_BASE_0 AS
SELECT A.TICKER, A.CURR_FPE, A.ESTIMATOR, A.ANALYS, A.CURR_EST_FIN_ANN, A.COUNTER_CURRFPE, A.LONG_EST,
B.MIN_VAL, B.MAX_VAL, B.POINT, B.ANNDATS AS MGMT_ANNDATS,
 (B.ANNDATS - A.CURR_EST_FIN_ANN) AS DAYS_BTW
FROM LONG_9 A LEFT JOIN MGMT_BASE B
ON A.TICKER=B.TICKER AND YEAR(A.COUNTER_CURRFPE) = B.PRD_YR AND MONTH(A.COUNTER_CURRFPE) = B.PRD_MON AND 
A.CURR_EST_FIN_ANN < B.ANNDATS;
QUIT;

DATA MGMT_BASE_1; SET MGMT_BASE_0; IF NOT MISSING(DAYS_BTW); RUN;

PROC SORT DATA = MGMT_BASE_1; 
BY TICKER CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN LONG_EST COUNTER_CURRFPE DAYS_BTW;QUIT;
PROC SORT DATA = MGMT_BASE_1 OUT = MGMT_BASE_2 NODUPKEY; 
BY TICKER CURR_FPE ESTIMATOR ANALYS CURR_EST_FIN_ANN LONG_EST COUNTER_CURRFPE;QUIT;

DATA MGMT_BASE_3; SET MGMT_BASE_2;
IF LONG_EST NE . THEN RLONG_EST = ROUND(LONG_EST, 0.01);
IF MISSING(MIN_VAL) = 0 AND MISSING(MAX_VAL) = 0 AND MISSING(POINT) = 1 THEN DO; 
IF RLONG_EST < MIN_VAL OR MAX_VAL < RLONG_EST THEN OUT_RANGE = 1;
END; 
IF MISSING(POINT) = 0 THEN DO; 
IF RLONG_EST < 0.95*POINT OR 1.05*POINT < RLONG_EST THEN OUT_RANGE = 1;
END;
IF OUT_RANGE NE 1 THEN OUT_RANGE = 0;
RUN;

PROC SQL;
CREATE TABLE LONG_10 AS
SELECT A.*, (B.OUT_RANGE = 1) AS OUT_RANGE
FROM LONG_9 A LEFT JOIN MGMT_BASE_3 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.ESTIMATOR = B.ESTIMATOR AND A.ANALYS = B.ANALYS AND 
A.CURR_EST_FIN_ANN = B.CURR_EST_FIN_ANN AND A.LONG_EST=B.LONG_EST AND A.COUNTER_CURRFPE = B.COUNTER_CURRFPE;
QUIT;

DATA LONG_11; SET LONG_10;
IF (PREV_ANN <= LONG_REVDATS & LONG_REVDATS <= PREV_ANN+10) OR (PREV_ANN <= CURR_EST_FIN_ANN & CURR_EST_FIN_ANN <= PREV_ANN+10) THEN LONG_REV10 = 1;
IF LONG_REV10 NE 1 THEN LONG_REV10 = 0;
RUN;

PROC EXPORT DATA = LONG_11
OUTFILE = "D:\IBES\result_202008\CROSS_LONG.DTA"
DBMS = STATA REPLACE;
QUIT;

/*******************************************************************************************/
*Managers' Trading;
*READ IN INSIDER TRADING DATA;
DATA TFN; SET RAW.INSIDER_TRADING;
IF  CLEANSE NOT IN ('S','A'); 
IF  TRANCODE IN ('P','S'); 
IF TRANDATE = .  THEN DELETE;
IF SHARES = .  THEN DELETE; 
IF TPRICE = .  THEN DELETE; 
IF MISSING(CUSIP6) = 1  THEN DELETE;
IF ROLECODE1 IN ('CB','CEO','CFO','CO','P');
YEAR = YEAR(TRANDATE);
RUN;

*IDENTIFY ALL THE TRANSACTIONS HAPPENED IN BETWEEN (EA_Q +1, EA_Q+1 -1);
PROC SQL;
CREATE TABLE ACT_22 AS
SELECT A.TICKER, A.CUSIP, A.PERMNO, A.USFIRM,
A.VALUE AS CURR_ACT, A.FPEDATS AS CURR_FPE, A.ANNDATS AS CURR_ANN, 
B.VALUE AS NEXT_ACT, B.FPEDATS AS NEXT_FPE, B.ANNDATS AS NEXT_ANN
FROM ACT_1 AS A, ACT_1 AS B
WHERE A.TICKER = B.TICKER 
AND  0 < B.FPEDATS - A.FPEDATS <= 110
AND  A.ANNDATS < B.ANNDATS;
QUIT; 
PROC SORT DATA = ACT_22; BY TICKER CURR_FPE CURR_ANN NEXT_FPE;QUIT;
PROC SORT DATA = ACT_22 OUT = ACT_23 NODUPKEY; BY TICKER CURR_FPE CURR_ANN;QUIT;
DATA ACT_23; SET ACT_23;
START_DT = CURR_ANN + 1;
END_DT = NEXT_ANN - 1;
FORMAT START_DT END_DT YYMMDD.;
RUN;

PROC SQL;
CREATE TABLE TFN1 AS
SELECT A.CUSIP6, A.DCN, MIN(A.TRANDATE) AS TRANDATE_START, MAX(A.TRANDATE) AS TRANDATE_END
FROM TFN A
GROUP BY A.DCN, A.CUSIP6;
QUIT;
DATA TFN1; SET TFN1;
FORMAT TRANDATE_START TRANDATE_END YYMMDD.;
RUN;

PROC SQL;
CREATE TABLE TFN2 AS 
SELECT A.TICKER, A.CUSIP, A.CURR_FPE, A.CURR_ANN, A.NEXT_ANN, A.NEXT_FPE,
B.DCN, B.TRANDATE_START, B.TRANDATE_END
FROM ACT_23 A LEFT JOIN TFN1 B
ON SUBSTR(A.CUSIP,1,6)=B.CUSIP6 AND A.START_DT <= B.TRANDATE_START AND B.TRANDATE_END <= A.END_DT;
QUIT;

DATA TFN3; SET TFN2;
IF MISSING(DCN) = 0;
RUN;

PROC SQL;
CREATE TABLE TFN4 AS
SELECT A.*, B.PERSONID, B.SEQNUM, B.OWNER, B.ROLECODE1, B.TRANCODE, B.SHARES, B.TRANDATE
FROM TFN3 A LEFT JOIN TFN B
ON A.DCN = B.DCN;
QUIT;

PROC SQL;
CREATE TABLE SHARESHELD AS
SELECT A.DCN, B.TRANDATE, B.SHARESHELD, B.SEQNUM
FROM TFN4 A LEFT JOIN RAW.INSIDER_TRADING B
ON A.DCN = B.DCN;
QUIT;
PROC SQL;
CREATE TABLE SHARESHELD1 AS
SELECT A.DCN, MAX(A.TRANDATE) AS TRANDATE
FROM SHARESHELD A
GROUP BY A.DCN;

CREATE TABLE SHARESHELD2 AS
SELECT A.DCN, A.SHARESHELD, A.SEQNUM
FROM SHARESHELD A, SHARESHELD1 B
WHERE A.DCN = B.DCN AND A.TRANDATE = B.TRANDATE;
QUIT;
PROC SORT DATA = SHARESHELD2 NODUPREC; BY _ALL_; QUIT;
PROC SORT DATA = SHARESHELD2; BY DCN DESCENDING SEQNUM; QUIT;
PROC SORT DATA = SHARESHELD2 OUT = SHARESHELD3 NODUPKEY; BY DCN; QUIT;
DATA SHARESHELD3; SET SHARESHELD3; IF SHARESHELD = . THEN SHARESHELD = 0; RUN;

PROC SQL;
CREATE TABLE TFN5 AS
SELECT  A.*, B.SHARESHELD
FROM TFN4 A LEFT JOIN SHARESHELD3 B
ON A.DCN = B.DCN;
QUIT;

DATA TFN5; SET TFN5;
IF TRANCODE = "P" AND SHARESHELD = 0 THEN FLAG = 1; ELSE FLAG = 0;
RUN;

PROC MEANS DATA = TFN5 MEAN;
VAR FLAG;
QUIT;

DATA TFN6; SET TFN5;
IF MISSING(TRANCODE) = 0;
IF TRANCODE = "S" THEN SALE = 1;
IF TRANCODE = "P" THEN SALE = -1;
RUN;
PROC SORT DATA = TFN6 NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE TFN7 AS
SELECT A.TICKER, A.CUSIP, A.CURR_FPE, A.CURR_ANN, A.NEXT_ANN, A.NEXT_FPE,
A.ROLECODE1, A.PERSONID, A.OWNER, A.SHARESHELD,
SUM(A.SHARES*A.SALE) AS SHARES_SOLD
FROM TFN6 A
WHERE A.TRANDATE <= A.CURR_ANN + 7
GROUP BY A.TICKER, A.CUSIP, A.CURR_FPE, A.CURR_ANN, A.NEXT_ANN, A.NEXT_FPE,
A.ROLECODE1, A.PERSONID, A.OWNER, A.SHARESHELD;
QUIT;

DATA TFN71; SET TFN7;
IF SHARES_SOLD < 0 AND SHARESHELD < SHARES_SOLD*(-1) THEN DO;
SHARES_SOLD_ADJ = SHARES_SOLD;
SHARES_HELD_ADJ = SHARES_SOLD*(-1);END;
IF SHARES_SOLD_ADJ = . THEN SHARES_SOLD_ADJ = SHARES_SOLD;
IF SHARES_HELD_ADJ = . THEN SHARES_HELD_ADJ = SHARESHELD;
IF SHARES_SOLD_ADJ > 0 AND SHARES_HELD_ADJ = 0 THEN SHARES_HELD_ADJ = SHARES_SOLD_ADJ;
RUN;

PROC SQL;
CREATE TABLE TFN8 AS
SELECT A.TICKER, A.CUSIP, A.CURR_FPE, A.CURR_ANN, A.NEXT_ANN, A.NEXT_FPE,
SUM(A.SHARES_SOLD_ADJ)/SUM(A.SHARES_HELD_ADJ) AS NET_SALE_SCALED
FROM TFN71 A
GROUP BY A.TICKER, A.CUSIP, A.CURR_FPE, A.CURR_ANN, A.NEXT_ANN, A.NEXT_FPE;
QUIT;

DATA MGR_BASE; SET RESULT.MAINSAMPLE_EPS; RUN;
PROC SQL;
CREATE TABLE MGR_BASE1 AS
SELECT  A.*, 
B.NET_SALE_SCALED 
FROM MGR_BASE A LEFT JOIN TFN8 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE AND A.CURR_ANN = B.CURR_ANN;
QUIT;
DATA MGR_BASE1; SET MGR_BASE1;
IF NET_SALE_SCALED = . THEN NET_SALE_SCALED = 0;
RUN;

/*Note: the variable NET_SALE_SCALED is converted into a dummy variable in STATA before the regression analysis

EGEN NETSALE_MEDIAN=XTILE(NET_SALE_SCALED), N(2)
REPLACE NETSALE_MEDIAN = NETSALE_MEDIAN - 1

*/

PROC EXPORT DATA= MGR_BASE1
OUTFILE='D:\IBES\RESULT_202008\CROSS_NETSALE.DTA' 
DBMS=STATA REPLACE; 
RUN;

*Compensation Delta;
/*MERGE IN CEO_DELTA */
DATA CEO; SET RAW.EXECU_ANNUAL_COMP;
IF CEOANN = 2;
GVKEY_N = GVKEY+0;
RUN;
DATA DELTA; SET RESULT.DELTA; GVKEY_N = GVKEY+0; RUN;

PROC SQL;
CREATE TABLE DELTA_1 AS
SELECT A.*, B.CEOANN
FROM RESULT.DELTA A LEFT JOIN CEO B
ON A.GVKEY_N = B.GVKEY_N AND A.YEAR = B.YEAR AND A.COPEROL = B.CO_PER_ROL;
QUIT;
DATA DELTA_2; SET DELTA_1; IF NOT MISSING(CEOANN);RUN;
/*Note: RESULT.DELTA is generated by Additional_Input.sas */

PROC SQL;
CREATE TABLE DELTA_3 AS
SELECT A.GVKEY, A.YEAR, MEAN(A.DELTA) AS DELTA
FROM DELTA_2 A
GROUP BY A.GVKEY,A.YEAR;
QUIT;

PROC SQL;
CREATE TABLE MGR_BASE_DELTA AS
SELECT A.*, B.GVKEY
FROM MGR_BASE AS A LEFT JOIN RAW.CCMXPF_LNKHIST AS B
ON A.PERMNO = B.LPERMNO AND 
B.LINKTYPE IN ("LC", "LS", "LU") AND
(B.LINKDT <= A.CURR_ANN OR MISSING(B.LINKDT) = 1) AND (A.CURR_ANN <= B.LINKENDDT OR B.LINKENDDT = .E);               

CREATE TABLE MGR_BASE_DELTA1 AS
SELECT A.*, B.DELTA
FROM MGR_BASE_DELTA A LEFT JOIN DELTA_3 B
ON A.GVKEY = B.GVKEY AND YEAR(A.CURR_FPE) = B.YEAR+1;
QUIT;

/*Note: the variable DELTA is converted into a dummy variable in STATA before the regression analysis

EGEN DELTA_MEDIAN=XTILE(DELTA), N(2)
REPLACE DELTA_MEDIAN = DELTA_MEDIAN - 1

*/

PROC EXPORT DATA= MGR_BASE_DELTA1
OUTFILE='D:\IBES\RESULT_202008\CROSS_DELTA.DTA' 
DBMS=STATA REPLACE; 
RUN;

*IR;
/*IMPORT IR DATA*/
PROC SORT DATA = RAW.IR (KEEP = GVKEY CONM IR RDQ FYEARQ FQTR) OUT = IR NODUPKEY;
BY GVKEY CONM IR RDQ FYEARQ FQTR;
QUIT;

DATA IR_MAIN_1; SET RESULT.MAINSAMPLE_EPS;
IF 2002 LE YEAR(CURR_FPE) < 2016;
RUN;

PROC SQL;
CREATE TABLE IR_MAIN_2 AS
SELECT A.*, B.GVKEY
FROM IR_MAIN_1 AS A LEFT JOIN RAW.CCMXPF_LNKHIST AS B
ON A.PERMNO = B.LPERMNO AND 
B.LINKTYPE IN ("LC", "LS", "LU") AND
(B.LINKDT <= A.CURR_ANN OR MISSING(B.LINKDT) = 1) AND (A.CURR_ANN <= B.LINKENDDT OR B.LINKENDDT = .E);               
QUIT;

PROC SQL;
CREATE TABLE IR_MAIN_3 AS
SELECT A.PERMNO, A.CURR_FPE, B.IR
FROM IR_MAIN_2 A LEFT JOIN IR B
ON A.GVKEY = B.GVKEY AND MISSING(B.GVKEY) = 0 AND  A.PREV_ANN - 110 <= B.RDQ <= A.PREV_ANN;
QUIT;
DATA IR_MAIN_4; SET IR_MAIN_3;
IF IR = 1;
RUN;
PROC SORT DATA = IR_MAIN_4 NODUPRECS; BY _ALL_ ; QUIT;

PROC SQL;
CREATE TABLE IR_MAIN_5 AS
SELECT A.*, (B.IR = 1) AS IR
FROM IR_MAIN_2 A LEFT JOIN IR_MAIN_4 B
ON A.PERMNO = B.PERMNO AND A.CURR_FPE = B.CURR_FPE;
QUIT;

PROC EXPORT DATA= IR_MAIN_5
OUTFILE='D:\IBES\RESULT_202008\CROSS_IR.DTA' 
DBMS=STATA REPLACE; 
RUN;

*Forecast Dispersion & Earnings Volatility;
DATA GUIDE_FOREBASE; SET RESULT.MAINSAMPLE_EPS; RUN;

PROC SQL;
CREATE TABLE GUIDE_FOREBASE_1 AS
SELECT A.*, LOG(B.VOL_END/A.PRC+1) AS FORE_DISPERSION
FROM GUIDE_FOREBASE A LEFT JOIN VOL_END B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

/*Note: the variable FORE_DISPERSION is converted into a dummy variable in STATA before the regression analysis

EGEN FORE_DISPERSION_MEDIAN=XTILE(FORE_DISPERSION), N(2)
REPLACE FORE_DISPERSION_MEDIAN = FORE_DISPERSION_MEDIAN - 1

*/

PROC EXPORT DATA= GUIDE_FOREBASE_1
OUTFILE='D:\IBES\RESULT_202008\CROSS_DISPERSION_FORECAST.DTA' 
DBMS=STATA REPLACE; 
RUN;

DATA BASE; SET RESULT.MAINSAMPLE_EPS;RUN;

DATA RAW.COMP_IBQ; SET RAW.COMP_8920_EAVOL; RUN;
DATA IBQ_BASE; SET RAW.COMP_8920_EAVOL; 
IF ATQ = 0 THEN DELETE;
IF ATQ < 0 THEN DELETE;
IF ATQ = . THEN DELETE;
RUN;
PROC SORT DATA = IBQ_BASE NODUPKEY; BY GVKEY FYEARQ FQTR; QUIT;

DATA IBQ_BASE; SET IBQ_BASE;
EARN1=IBQ/ATQ;
RUN;

PROC SORT DATA=IBQ_BASE (KEEP=GVKEY DATADATE FYEARQ FQTR EARN1) OUT=IBQ_BASE1 NODUPKEY;
BY GVKEY DATADATE FYEARQ FQTR EARN1;
QUIT;

PROC SQL;
CREATE TABLE EA_VOL AS
SELECT A.GVKEY, A.DATADATE, A.FYEARQ, A.FQTR,
B.DATADATE AS PAST_DATADATE,
B.FYEARQ AS PAST_FYEARQ,
B.FQTR AS PAST_FQTR,
B.EARN1
FROM IBQ_BASE1 A, IBQ_BASE1 B
WHERE A.GVKEY=B.GVKEY AND B.DATADATE BETWEEN A.DATADATE - 10 AND A.DATADATE - 1200;
QUIT;
PROC SORT DATA = EA_VOL OUT = EA_VOL_1 NODUPKEY; 
BY GVKEY DATADATE FYEARQ FQTR PAST_DATADATE EARN1;QUIT;

PROC SQL;
CREATE TABLE EA_VOL_2 AS
SELECT A.GVKEY,A.DATADATE,A.FYEARQ,A.FQTR,
STD(A.EARN1) AS EARN_VOL,MIN(A.PAST_DATADATE) AS MIN_DATADATE
FROM EA_VOL_1 A
GROUP BY A.GVKEY,A.DATADATE,A.FYEARQ,A.FQTR;
QUIT;
PROC SORT DATA = EA_VOL_2 NODUPKEY; 
BY GVKEY DATADATE FYEARQ FQTR EARN_VOL MIN_DATADATE;QUIT;

DATA EA_VOL_3; SET EA_VOL_2;
DIFF = DATADATE-MIN_DATADATE;
IF MISSING(EARN_VOL) = 1 THEN DELETE;
RUN;
PROC SORT DATA = EA_VOL_3 NODUPKEY; 
BY GVKEY DATADATE FYEARQ FQTR;QUIT;

/*MERGE IN PERMNO*/
PROC SQL;
CREATE TABLE EA_VOL_4 AS
SELECT A.*, B.LPERMNO AS PERMNO
FROM EA_VOL_3 AS A LEFT JOIN RAW.CCMXPF_LNKHIST AS B
ON A.GVKEY = B.GVKEY AND 
B.LINKTYPE IN ("LC", "LS", "LU") AND
(B.LINKDT <= A.DATADATE OR MISSING(B.LINKDT) = 1) AND (A.DATADATE <= B.LINKENDDT OR B.LINKENDDT = .E);               
QUIT;
DATA EA_VOL_41; SET EA_VOL_4 (RENAME=(PERMNO=PERMNO_CHAR));
IF MISSING(PERMNO_CHAR)=0;
PERMNO=INPUT(PERMNO_CHAR,32.);
DROP PERMNO_CHAR;
RUN;
PROC SORT DATA = EA_VOL_41 OUT = EA_VOL_5 NODUPKEY;
BY PERMNO DATADATE;
QUIT;

PROC SQL;
CREATE TABLE EA_VOL_6 AS
SELECT A.*, B.EARN_VOL, B.DIFF
FROM BASE AS A LEFT JOIN EA_VOL_5 AS B
ON A.PERMNO = B.PERMNO AND YEAR(A.CURR_FPE) = YEAR(B.DATADATE) AND MONTH(A.CURR_FPE)=MONTH(B.DATADATE);
QUIT;

PROC EXPORT DATA= EA_VOL_6
FILE='D:\IBES\RESULT_202008\CROSS_EAVOL.DTA' 
DBMS=STATA REPLACE; 
RUN;

DATA MGMT (KEEP = TICKER ANNDATS MEASURE PDICITY PRD_MON PRD_YR MGMT_DUMMY DOWN_DUMMY); SET RAW.MGMT_92_18;
MGMT_DUMMY = 1;
IF MEASURE = "EPS";
IF MISSING(VAL_1) = 0 OR MISSING(VAL_2) = 0;
IF PDICITY = "QTR";
IF GUIDANCE_CODE = "01" THEN DOWN_DUMMY = 1; ELSE DOWN_DUMMY = 0;
RUN;

PROC SORT 
DATA = RESULT.MAINSAMPLE_EPS (KEEP = TICKER CURR_FPE CURR_ANN PREV_ANN CURR_ACT) OUT = PREV_ANN NODUPKEY;
BY TICKER CURR_FPE CURR_ANN PREV_ANN;
QUIT;

PROC SQL;
CREATE TABLE MGMT_1 AS
SELECT A.*, B.CURR_ANN, B.PREV_ANN, B.CURR_FPE, B.CURR_ACT
FROM MGMT A LEFT JOIN PREV_ANN B
ON A.TICKER = B.TICKER AND A.PRD_MON = MONTH(B.CURR_FPE) AND A.PRD_YR = YEAR(B.CURR_FPE);
QUIT;

DATA MGMT_2; SET MGMT_1;
IF ANNDATS > PREV_ANN + 5;
IF ANNDATS <   CURR_ANN;
RUN;

DATA MGMT_2; SET MGMT_2;
IF DOWN_DUMMY = 1 THEN ANNDATS_DOWN = ANNDATS; ELSE ANNDATS_DOWN = .;
FORMAT ANNDATS_DOWN YYMMDD.;
IF ANNDATS >= CURR_FPE THEN MGMT_FPD = 1; ELSE MGMT_FPD = 0;
RUN;

PROC SQL;
CREATE TABLE MGMT_3 AS
SELECT A.TICKER, A.CURR_FPE, A.CURR_ANN, A.PREV_ANN, A.CURR_ACT,
SUM(A.DOWN_DUMMY) AS DOWN_DUMMY
FROM MGMT_2 A
GROUP BY A.TICKER, A.CURR_FPE, A.CURR_ANN, A.PREV_ANN, A.CURR_ACT;
QUIT;
DATA MGMT_4; SET MGMT_3;
DOWN_DUMMY = (DOWN_DUMMY > 0);
RUN; 

DATA BASE; SET RESULT.MAINSAMPLE_EPS; RUN;

DATA DET_1 (KEEP = TICKER ANALYS FPEDATS EST ANNDATS REVDATS FPI ESTIMATOR ANNTIMS ACTDATS); 
SET RAW.IBES_DETAIL_QTR;
EST = VALUE;
IF FPI IN ('6','7','8','9','N','O','P','Q','R','S','T','L','Y');
IF MEASURE = 'EPS';
IF TICKER NE "";
IF ANALYS NE .;
IF EST NE .;
RUN;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS DESCENDING ANNTIMS; QUIT;
PROC SORT DATA = DET_1 NODUPKEY; BY TICKER ANALYS FPEDATS ANNDATS; QUIT;

PROC SQL;
CREATE TABLE OPT_EA0 AS
SELECT A.TICKER, A.PERMNO, A.ACT_ADJ, A.CURR_FPE, B.ESTIMATOR, B.ANALYS, B.ANNDATS, B.EST
FROM BASE A LEFT JOIN DET_1 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.FPEDATS AND  B.ANNDATS <= A.PREV_ANN + 5;
QUIT;
PROC SORT DATA = OPT_EA0; BY TICKER CURR_FPE ESTIMATOR DESCENDING ANNDATS; QUIT;
PROC SORT DATA = OPT_EA0 OUT = OPT_EA1 NODUPKEY; BY TICKER CURR_FPE ESTIMATOR; QUIT;

DATA RET; SET RAW.DAILY_RET;
KEEP PERMNO PRC CFACSHR DATE;
RUN;
PROC SORT DATA = RET NODUPREC; BY _ALL_; QUIT;

PROC SQL;
CREATE TABLE DET_ADJ AS
SELECT A.TICKER, A.PERMNO, A.CURR_FPE, A.ACT_ADJ, A.ESTIMATOR, A.ANALYS, A.ANNDATS, A.EST, B.CFACSHR, B.DATE AS CRSP_DATE
FROM OPT_EA1 A LEFT JOIN RET B
ON A.PERMNO = B.PERMNO AND 3 >= A.ANNDATS -  B.DATE >= 0;
QUIT;

PROC SORT DATA = DET_ADJ; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS ANNDATS DESCENDING CRSP_DATE;QUIT;
PROC SORT DATA = DET_ADJ OUT = DET_ADJ1 NODUPKEY; BY TICKER PERMNO CURR_FPE ESTIMATOR ANALYS ANNDATS;QUIT;

DATA DET_ADJ2; SET DET_ADJ1;
IF MISSING(ACT_ADJ) = 0 AND MISSING(CFACSHR) = 0 AND (ACT_ADJ NE CFACSHR) THEN DO;
EST = EST*(ACT_ADJ/CFACSHR);
END;
RUN;

PROC SQL;
CREATE TABLE OPT_EA2 AS
SELECT A.TICKER, A.CURR_FPE, MEDIAN(A.EST) AS OPT_MEDIAN, STD(A.EST) AS STD_FORE, COUNT(DISTINCT A.ESTIMATOR) AS FOLLOWING
FROM DET_ADJ2 A
GROUP BY A.TICKER, A.CURR_FPE;
QUIT;

PROC SQL;
CREATE TABLE ATLEASTONE AS
SELECT A.TICKER, A.CURR_FPE, A.CURR_ACT, A.PRC, A.PERMNO, SUM(A.FL_EXCL) AS TOTAL_REMOVAL
FROM BASE A
GROUP BY A.TICKER, A.CURR_FPE, A.CURR_ACT, A.PRC, A.PERMNO;
QUIT;

DATA ATLEASTONE1; SET ATLEASTONE;
IF TOTAL_REMOVAL > 0 THEN ATLEASTONE = 1; ELSE ATLEASTONE = 0;
RUN;

PROC SQL;
CREATE TABLE SAMPLE AS
SELECT A.*, B.DOWN_DUMMY
FROM ATLEASTONE1 A LEFT JOIN MGMT_4 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;

CREATE TABLE SAMPLE1 AS
SELECT A.*, B.OPT_MEDIAN, B.STD_FORE, B.FOLLOWING
FROM SAMPLE A LEFT JOIN OPT_EA2 B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

DATA SAMPLE3; SET SAMPLE1;
IF MISSING(DOWN_DUMMY) = 1 THEN DOWN_DUMMY = 0;
RUN;

PROC IMPORT OUT= BASE_EAVOL
FILE='D:\IBES\RESULT_202008\CROSS_EAVOL.DTA'
DBMS=DTA REPLACE;
RUN;

PROC SORT DATA = BASE_EAVOL (KEEP = TICKER CURR_FPE EARN_VOL DIFF) OUT = BASE_EAVOL NODUPKEY;
BY TICKER CURR_FPE EARN_VOL DIFF;
QUIT;

PROC SQL;
CREATE TABLE SAMPLE4 AS
SELECT A.*, B.EARN_VOL, B.DIFF
FROM SAMPLE3 AS A LEFT JOIN BASE_EAVOL AS B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

DATA SAMPLE4; SET SAMPLE4;
LOSS = (CURR_ACT < 0 );
LAST_STATPERS = INTNX('MONTH', CURR_FPE, -18, "SAMEDAY");
FORMAT LAST_STATPERS YYMMDD.;
RUN;

DATA COMP; SET RAW.COMP_9020_QTRFIRMCHAR;
IF FQTR =  1 THEN LAG1QTR = (FYEARQ-1)*10+4;
IF FQTR ~= 1 THEN LAG1QTR = FYEARQ*10+FQTR-1;
RUN;
PROC SORT DATA = COMP NODUPREC;BY   _ALL_; QUIT;

PROC SORT DATA = COMP (KEEP = GVKEY FYEARQ FQTR ATQ) OUT = ATQ NODUPKEY; BY     GVKEY FYEARQ FQTR ATQ; QUIT;
PROC SQL;
CREATE TABLE COMP1 AS
SELECT A.*, B.ATQ AS LAG1QTR_ATQ
FROM COMP A LEFT JOIN ATQ B
ON   A.GVKEY = B.GVKEY AND A.LAG1QTR = B.FYEARQ*10+B.FQTR;
QUIT;

DATA COMP2; SET COMP1;
MVE 	= LOG(CSHOQ*PRCCQ);
BOOK 	= CEQQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN BOOK = SEQQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN BOOK = ATQ - LTQ - SUM(TXDBQ,PSTKQ);
IF BOOK = . THEN DELETE;
GP = (REVTQ - COGSQ)/ATQ;  
IF GP = . THEN GP = REVTQ/ATQ;
OP = SUM(REVTQ,-COGSQ,-XSGAQ,-XINTQ,-XRDQ)/BOOK; 
CMA 	= (ATQ - LAG1QTR_ATQ)/LAG1QTR_ATQ;  
IF LAG1QTR_ATQ = . THEN DELETE;
IF LAG1QTR_ATQ LE 0 THEN DELETE;
IF ATQ = . THEN DELETE;
IF ATQ LE 0 THEN DELETE;
IF GP = . THEN DELETE;
BTM = BOOK/(CSHOQ*PRCCQ);
RUN;

PROC SQL;
CREATE TABLE COMP4 AS
SELECT A.*, B.LPERMNO AS PERMNO
FROM   COMP2 A LEFT JOIN RAW.CCMXPF_LNKHIST AS B
ON     A.GVKEY = B.GVKEY 	 AND 
B.LINKTYPE IN ("LC","LS","LU") 	  AND 
(B.LINKDT <= A.DATADATE OR MISSING(B.LINKDT) = 1)    AND 
(A.DATADATE <= B.LINKENDDT OR B.LINKENDDT = .E);               
QUIT;

PROC SQL;
CREATE TABLE FIRM_CHAR AS
SELECT A.TICKER, A.CURR_FPE, B.GVKEY, B.DATADATE, B.GP, B.OP, B.CMA, B.BTM, B.MVE, B.SIC
FROM SAMPLE4 A LEFT JOIN  COMP4 B
ON A.PERMNO = B.PERMNO AND  A.LAST_STATPERS <= B.DATADATE < A.CURR_FPE;
QUIT;
PROC SORT DATA = FIRM_CHAR; BY TICKER CURR_FPE DESCENDING DATADATE; QUIT;
PROC SORT DATA = FIRM_CHAR NODUPKEY; BY TICKER CURR_FPE; QUIT;

PROC SQL;
CREATE TABLE SAMPLE5 AS
SELECT A.*, B.GP, B.OP as ROA, B.CMA, B.BTM, B.MVE, B.SIC
FROM SAMPLE4 A LEFT JOIN FIRM_CHAR B
ON A.TICKER = B.TICKER AND A.CURR_FPE = B.CURR_FPE;
QUIT;

DATA SAMPLE6; SET SAMPLE5;
OPT_MEDIAN = ROUND(OPT_MEDIAN, 0.01);
OPT_FIRM = (OPT_MEDIAN > CURR_ACT);
SIC_NUM = SIC + 0;
SIC2_NUM = int(SIC/100);
YEAR_QTR= YEAR(CURR_FPE)*10+QTR(CURR_FPE);
RUN;

PROC EXPORT 
DATA= SAMPLE6
FILE='D:\IBES\RESULT_202008\EAvol_STDFORE_FIRMQTR.DTA' 
DBMS=STATA REPLACE; 
RUN;

/*Note: All the regression tests are conducted in STATA.*/
